
/* memusage.c

A more detailed 'sin' that can show page table entries used..
*/


#include <libc.h>
#include <sys/kernel.h>
#include <sys/psinfo.h>
#include <sys/proc_msg.h>

typedef struct Proc Proc;
typedef struct _psinfo Psinfo;

struct Proc {
	union { Psinfo; Psinfo Psinfo;};
	Proc   *mids;
	Proc   *vids;
	Proc   *vmids;
	Proc   *code;
	Proc   *next;
};

#define ps_dpages(p) (*(ulong_t *)(p)->zero1)
#define ps_cpages(p) (*(ulong_t *)(p)->un.proc.zero2)

Proc   *proclist;
Proc   *vidlist;
Proc   *midlist;
Proc   *vmidlist;
ulong_t memtot;



Proc *
newproc()
{
	Proc  *p;
	if ((p=calloc(sizeof *p,1)) == 0) {
		fprintf(stderr, "calloc: nomemory\n");
		exit(1);
	}
	return p;
}

readprocs(pid_t proc)
{
	struct _psinfo psdata;
	Proc          *p;
	int            pi;
	for (pi=1 ; (pi=qnx_psinfo(proc, pi, &psdata, 0, 0)) != -1; pi++) {
		if (psdata.state == STATE_DEAD)
			continue;
		p = newproc();
		p->Psinfo = psdata;
		switch (p->flags & (_PPF_MID|_PPF_VID)) {
		case 0:
			p->next = proclist;
			proclist = p;
			break;
		case _PPF_VID:
			p->next = vidlist;
			vidlist = p;
			break;
		case _PPF_MID:
			p->next = midlist;
			midlist = p;
			break;
		case _PPF_MID|_PPF_VID:
			p->next = vmidlist;
			vmidlist = p;
			break;
		}
	}
}

sortprocs()
{
	Proc     *p, *v, *m;
	while (v=vidlist) {
		vidlist = v->next;
		for (p=proclist; p; p=p->next) {
			if (p->pid == v->un.vproc.local_pid) {
				v->next = p->vids;
				p->vids = v;
				break;
			}
		}
	}
	while (v=vmidlist) {
		vmidlist = v->next;
		for (p=proclist; p; p=p->next) {
			if (p->pid == v->un.vproc.local_pid) {
				v->next = p->vmids;
				p->vmids = v;
				break;
			}
		}
	}
	while (m=midlist) {
		midlist = m->next;
		for (p=proclist; p; p=p->next) {
			if (p->pid == m->blocked_on) {
				m->next = p->mids;
				p->mids = m;
				break;
			}
		}
	}
	m = proclist;
	proclist = 0;
	while (v=m) {
		m=m->next;
		for (p=m; p; p=p->next) {
			if (strcmp(p->un.proc.name, v->un.proc.name) == 0) 
				break;
		}
		if (p) {
			v->next = p->code;
			p->code = v;
		}  else {
			v->next = proclist;
			proclist = v;
		}
	}
}


char *
_state(int state, int substate)
{
	switch (state) {
	case STATE_DEAD: return "dead";
	case STATE_READY: return "ready";
	case STATE_SEND_BLOCKED: return "sending";
	case STATE_RECEIVE_BLOCKED: return "receiving";
	case STATE_HELD: return "held";
	case STATE_SIGNAL_BLOCKED:
	case STATE_REPLY_BLOCKED:
		switch (substate) {
		case 0: return state == STATE_REPLY_BLOCKED? "reply":"signal";
		case _VCS_READY: return "idle or replying";
		case _VCS_VA_NET_BLK: return "attaching";
		case _VCS_VAS_NET_BLK: return "shared attaching";
		case _VCS_GS_PROC_BLK: return "send-growing local";
		case _VCS_GS_NET_BLK:   return "send-growing remote";
		case _VCS_S_NET_BLK:   return "sending";
		case _VCS_GR_PROC_BLK:  return "reply-growing local";
		case _VCS_GR_NET_BLK:   return "reply-growing remote";
		case _VCS_GW_PROC_BLK:  return "writemsg-growing local";
		}
		return "unknown";
	case STATE_WAIT_BLOCKED:
		return "waiting";
	case STATE_SEM_BLOCKED:
		return "semaphore";
	default:
		return "unknown";
	}
}

print_proc(Proc *proc)
{
	ulong_t  code=0, data=0, shared=0;
	data = ps_dpages(proc);
	if ((proc->flags & (_PPF_MID|_PPF_VID)) == 0) {
		code = ps_cpages(proc);
	}
	printf("% 5u@%u % 5u % 5u\n", proc->pid, proc->max_priority, code, data);
}


print_vid(Proc *j)
{
	printf("\t\tVID: % 5u@%u -> %lu:%u (%u pages %u links '%s')\n",
		j->pid,
		j->max_priority,
		j->un.vproc.remote_nid,
		j->un.vproc.remote_pid,
		ps_dpages(j),
		j->un.vproc.links,
		_state(j->state, j->un.vproc.substate));
}

dproc(Proc *p)
{
	Proc   *j;
	printf("\tPROC: % 5u@%u '%s', ldt=0x%x, %u pages\n", p->pid, p->max_priority,
		_state(p->state,0), p->ldt, ps_dpages(p));
	memtot += ps_dpages(p);
	if (p->vids) {
		for (j=p->vids; j; j=j->next) {
			print_vid(j);
			memtot += ps_dpages(j);
		}
	}
	if (p->mids) {
		printf("\t\tPROXY: ");
		for (j=p->mids; j; j=j->next) {
			printf("% 5u%s", j->pid, j->next?", ":"\n");
			memtot += ps_dpages(j);
		}
	}
	if (p->vmids) {
		printf("\t\tVPROXY: ");
		for (j=p->vmids; j; j=j->next) {
			printf("% 5u%s", j->pid, j->next?", ":"\n");
			memtot += ps_dpages(j);
		}
	}
}

dump()
{
	Proc   *t;
	Proc   *p;
	for (t=proclist; t; t=t->next) {
		printf("%s %u code pages\n",
			t->un.proc.name, ps_cpages(t));
		memtot += ps_cpages(t);
		dproc(t);
		for (p=t->code; p; p=p->next)
			dproc(p);
	}
}


getmeml(pid_t proc)
{
	struct _proc_check   p;
	struct _proc_check_reply r;

	memset(&p, 0, sizeof p);
	p.type = _PROC_CHECK;
	p.subtype = _PROC_SUB_MEMCHECK;
	Send(proc, &p, &r, sizeof p, sizeof r);
	if (r.status == EOK) {
		printf("pmem: free %u count %u tot %u used %u\n",
			r.data[0], r.data[1], r.data[2], r.data[2]-r.data[1]);
		printf("vmem: free %u count %u tot %u used %u\n",
			r.data[3], r.data[4], r.data[5], r.data[5]-r.data[3]);
	}
}

main(int argc, char **argv)
{
	int     c;
	nid_t   nid = 0;
	pid_t   p;
	while ((c=getopt(argc, argv, "n:")) != -1) {
		if (c == 'n') {
			nid = strtol(optarg, 0, 0);
		} else {
			exit(1);
		}
	}
	if ((p = qnx_vc_attach(nid, PROC_PID, sizeof(Proc), 1)) == -1) {
		fprintf(stderr, "cannot attach to nid %u\n", nid);
		exit(1);
	}
	readprocs(p);
	sortprocs();
	dump();
	getmeml(p);
	printf("memtot = %u\n", memtot);
	return 0;
}

