/*- $Id: page.h,v 423.4 1997/12/10 17:06:20 steve Exp $
 * intel pager
 *  (intel-ese): pte is a page table entry.  Pte's exist in tables of 1024,
 * which are pointed to by pde's.  Pde's look just like pte's.
 * Each page table is one page long, and covers 2^22 bytes of address space.
 * The physical address of the page directory is loaded onto the processor.
 *
 * For convenience, we make the last pde point to the page directory, thus
 * the upper 4M of address space provide a nice linear array of pte's.
 * The pte for virtual address 0, is thus *0xffc00000.
 * Similiarly, the upper 4K of address space (0xfffff000..0xfffffff) are
 * a handy way to get at the page directory itself.  The pde for address 0
 * is at 0xfffff000, the pde for address 0x400000 is at 0xfffff004.

Selector 0x20 points at virtual address 0, limit = 2^32-1.
Selector 0x8 points at the gdt itself.
A process's LDT points into the gdt at a greater offset.
This program works on 423, 424 and at least some versions of 425.

*/

#include <libc.h>
#include <i86.h>

typedef struct {
	unsigned    present:1,
	                   :11,
	            pageno:20;
} pte_t;

typedef struct {
	unsigned    lim0:16,
	            base0:16,
	            base1:8,
	            type:4,
	            seg:1,
	            dpl:2,
	            present:1,
	            lim1:4,
	            notalloc:1,
	            :1,
	            big:1,
	            large:1,
	            base2:8;
} seg_t;

typedef struct {
	unsigned    lim0:16,
	            base0:16,
	            base1:8,
	            type:4,
	            gate:1,
	            dpl:2,
	            present:1,
	            lim1:4,
	            :3,
	            large:1,
	            base2:8;
} gate_t;

#define itosel(i) ((i)*sizeof(seg_t))
#define seltoi(s) ((s)/sizeof(seg_t))

pte_t __far *pdir; /*loopback pdirs */
pte_t __far *ptab; /*loopback ptes */
seg_t __far *gdt;  /*loopback gdt*/
paddr_t      gdt_base, gdt_end;
unsigned     ngdt;

sload(seg_t __far *p, paddr_t *b, paddr_t *e)
{
	*b = p->base0 + (p->base1<<16) + (p->base2 << 24);
	*e = (p->lim0 + (p->lim1<<16)) << (p->large?12:0);
	*e += *b;
}

insel(seg_t __far *gdt, unsigned selno, paddr_t vaddr)
{
	paddr_t b,e;
	if (selno >= ngdt || gdt[selno].present == 0 ||
		gdt[selno].seg == 0
		/*|| gdt[selno].notalloc*/) {
		return 0;
	}
	sload(gdt+selno, &b, &e);

	if (vaddr >= b && vaddr <= e)
		return selno;
	return 0;
}

unmap_ldt(seg_t __far *gdt, unsigned ldtno, unsigned selno)
{
	paddr_t b, e;
	unsigned l0, ln;
	if (ldtno >= ngdt || gdt[ldtno].present == 0 ||
	    gdt[ldtno].seg || gdt[ldtno].type != 0x2) {
	   	return -1;
	}
	sload(gdt+ldtno, &b, &e);
	l0 = seltoi(b-gdt_base);
	ln = seltoi(e-gdt_base);
	if (selno >= l0 && selno <= ln)
		return selno - l0;
	return -1;
}


page_find(paddr_t A, int (*f)(), void *arg)
{
	unsigned dir, page, sel;
	long  pageno;

	for (dir=0; dir < 1023; dir++) {
		if (pdir[dir].present == 0)
			continue;
		for (page=0; page < 1023; page++) {
			if (ptab[dir*1024+page].present == 0)
				continue;
			if (ptab[dir*1024+page].pageno == (A>>12)) {
				f((dir*1024+page)<<12, arg);
			}
		}
	}
}

seg_find(paddr_t v, int (*f)())
{
	unsigned sel;
	unsigned found = 0;
	printf("vaddr %x: ", v);
	for (sel=1; sel < ngdt; sel++) {
		if (!insel(gdt, sel, v))
			continue;
		f(sel);
		found++;
	}
	if (!found) printf("\n");
}

ldt_find(unsigned selno)
{
	unsigned i,j;
	unsigned found = 0;

	printf("selector %x: ", itosel(selno));
	for (i=32; i < ngdt; i++) {
		if ((j=unmap_ldt(gdt, i, selno)) != -1) {
			printf("proc index %u owns selector %x as %u\n",
				i-32+1, itosel(selno), itosel(j)|4);
			found++;
		}
	}
	if (!found) printf("\n");
}


main()
{
	char      buf[80];
	pdir= MK_FP(0x20,0xfffff000); /*loopback pdirs */
	ptab= MK_FP(0x20,0xffc00000); /*loopback ptes */
	gdt = MK_FP(0x8,0); /*loopback gdt*/

	sload(gdt+1, &gdt_base, &gdt_end);
	ngdt = seltoi(gdt_end-gdt_base);


	while (gets(buf)) {
		paddr_t p = strtol(buf, 0, 0);
		page_find(p, seg_find, ldt_find);
	}
}

