/*-
 * This event monitor separates execution times by interrupt and priority.
 * interrupts are managed by a stack; the bottom-most entry is the process.
 * accum[asp] points to the current accumulator, which is either in the
 * intrtab during interrupt processing or priotab during process execution.
 * Interrupts nest naturally via the accum stack.  Process priorities are
 * tracked in prio[].
 */
#include "imon.h"

extern l64 *priotab;
extern l64 *intrtab;
static char prio[4096];

enum  {NNEST = 32};
static l64 last, unknown;
static l64 *accum[NNEST] = { &unknown };
static      asp;
int         curprio;
static int  curpid;

_evstart(pid_t pid)
{
	curpid   = pid;
	curprio  = prio[PINDEX(curpid)];
}

int
EvAp_put()
{
	unsigned id;

	ul_add_diff(accum[asp], monevent.timestamp, last);
	ul_copy(last, monevent.timestamp);

	switch (EOPCODE(monevent.code)) {
	case _EVENT_KER_IRQ:
		accum[++asp] = intrtab+monevent.ev.eventp->irq.irq;
		break;
	case _EVENT_KER_IRQDONE:
		asp--;
		break;
	case _EVENT_KER_ACTIVE:
		curprio = prio[curpid = PINDEX(EDATA(monevent.code))];
		accum[asp] = priotab+curprio;
		break;
	case _EVENT_KER_PRIORITY:
		prio[id=PINDEX(EDATA(monevent.code))] = monevent.ev.eventp->prio.priority;
		if (id == curpid) {
			curprio = prio[id];
			accum[asp] = priotab+curprio;
		}
		break;
	case _EVENT_KER_PROC_STAT:
		prio[id=PINDEX(EDATA(monevent.code))] = monevent.ev.eventp->stat.priority;
		if (id == curpid) {
			curprio = prio[id];
			accum[asp] = priotab+curprio;
		}
		break;
	}
	return 0;
}
