/*-
 *  kernel.h    kernel function calls/pragmas
 *
 *  Copyright by WATCOM International Corp. 1988-1993.  All rights reserved.
 */
#ifndef __KERNEL_H_INCLUDED
#define __KERNEL_H_INCLUDED

#ifndef __TYPES_H_INCLUDED
 #include <sys/types.h>
#endif

#ifndef __SENDMX_H_INCLUDED
 #include <sys/sendmx.h>
#endif

#define PROC_PID    1

/*
 * Task States
 */

#define STATE_DEAD              0
#define STATE_READY             1
#define STATE_SEND_BLOCKED      2
#define STATE_RECEIVE_BLOCKED   3
#define STATE_REPLY_BLOCKED     4
#define STATE_HELD              5
#define STATE_SIGNAL_BLOCKED    6
#define STATE_WAIT_BLOCKED      7
#define STATE_SEM_BLOCKED       8

/*
 *  virtual processes substates.  Note that the vid's state must be
 *  reply blocked on 0 for any of these blocked substates to be valid
 */
#define _VCS_READY          1   /* idle (could be txing reply, also)    */
#define _VCS_VA_NET_BLK     2   /* initially attaching, blkd on network */
#define _VCS_VAS_NET_BLK    3   /* shared attaching, blkd on network    */
#define _VCS_GS_PROC_BLK    4   /* growing on send, blkd on local proc  */
#define _VCS_GS_NET_BLK     5   /* growing on send, blkd on network     */
#define _VCS_S_NET_BLK      6   /* sending, blkd on network (normal)    */
#define _VCS_GR_PROC_BLK    7   /* growing on reply, blkd on local proc */
#define _VCS_GR_NET_BLK     8   /* growing on reply, blkd on network    */
#define _VCS_GW_PROC_BLK    9   /* growing on writemsg, blkd on proc    */


/*- kernel call #s */
enum {
 __KER_SEND    = 0,
 __KER_RECV    = 1,
 __KER_REPLY   = 2,
 __KER_CRECV   = 3,
 __KER_READ    = 4,
 __KER_WRITE   = 5,
 __KER_RELAY   = 6,
 __KER_SIGRET  = 7,
 __KER_PRIO    = 8,
 __KER_NET     = 9,
 __KER_YIELD   = 10,
 __KER_SENDFD  = 11,
 __KER_TRIGGER = 12,
 __KER_KILL    = 14,
 __KER_SPECIAL = 13,
 __KER_SIGSUSP = 14+0x100,
 __KER_APM     = 15,
 __KER_SEMOP   = 16,
 __KER_MEM     = 17,
 __KER_SPLICE  = 18,
 __KER_AUX     = 19
};

#if !defined __WATCOMC__ || defined __386__

#ifdef __cplusplus
extern "C" {
#endif

#define Send( a,b,c,d,e ) __send( a,b,c,d,e )
#define Sendfd( a,b,c,d,e ) __sendfd( a,b,c,d,e )
#define Receive( a,b,c ) __receive( a,b,c )
#define Reply( a,b,c ) __reply( a,b,c )
#define Creceive( a,b,c ) __creceive( a,b,c )
#define Readmsg( a,b,c,d ) __readmsg( a,b,c,d )
#define Writemsg( a,b,c,d ) __writemsg( a,b,c,d )

extern int      __send(pid_t __pid, const void *__msg1, void *__msg2,
                       unsigned __nbytes1, unsigned __nbytes2);
extern int      __sendfd(int __fd, const void *__msg1, void *__msg2,
                         unsigned __nbytes1, unsigned __nbytes2);
extern pid_t    __receive(pid_t __pid, void *__msg, unsigned __nbytes);
extern int      __reply(pid_t __pid, const void *__msg, unsigned __nbytes);
extern pid_t    __creceive(pid_t __pid, void *__msg, unsigned __nbytes);
extern unsigned __readmsg(pid_t __pid, unsigned __offset, void *__msg,
                          unsigned __nbytes);
extern unsigned __writemsg(pid_t __pid, unsigned __offset, const void *__msg,
                          unsigned __nbytes);
extern int      __kererr(int errcode);

#define Sendmx( a,b,c,d,e ) __sendmx( a,b,c,d,e )
#define Receivemx( a,b,c ) __receivemx( a,b,c )
#define Replymx( a,b,c ) __replymx( a,b,c )
#define Creceivemx( a,b,c ) __creceivemx( a,b,c )
#define Readmsgmx( a,b,c,d ) __readmsgmx( a,b,c,d )
#define Writemsgmx( a,b,c,d ) __writemsgmx( a,b,c,d )
#define Relay( a,b ) __relay( a,b )
#define Kill( a,b ) __kill( a,b )
#define Sigsuspend( a ) __sigsuspend( a )
#define __Sret() __sret()
#define Priority( a,b ) __priority( a,b )
#define Netdata( a,b ) __netdata( a,b )
#define Yield() __yield()
#define Sendfdmx( a,b,c,d,e ) __sendfdmx( a,b,c,d,e )
#define Trigger(a) __trigger(a)
#define Semaphore( a,b ) __semaphore( a,b )

extern int __sendmx(
            int                   __pid,
            unsigned              __smsg_xparts,
            unsigned              __rmsg_xparts,
            const struct _mxfer_entry  *__smsg,
            const struct _mxfer_entry  *__rmsg );

extern int __receivemx(
            pid_t                 __pid,
            unsigned              __rmsg_xparts,
            struct _mxfer_entry  *__rmsg );

extern int __replymx(
            pid_t                 __pid,
            unsigned              __rmsg_xparts,
            struct _mxfer_entry  *__rmsg );

extern int __creceivemx(
            pid_t                 __pid,
            unsigned              __rmsg_xparts,
            struct _mxfer_entry  *__rmsg );

extern unsigned __readmsgmx(
            pid_t                 __pid,
            unsigned              __offset,
            unsigned              __rmsg_xparts,
            struct _mxfer_entry  *__rmsg);

extern unsigned __writemsgmx(
            pid_t                 __pid,
            unsigned              __offset,
            unsigned              __wmsg_xparts,
            struct _mxfer_entry  *__wmsg );

extern int __relay(pid_t __pid1, pid_t __pid2);

extern int __kill(pid_t __pid, int signo);

extern int __sigsuspend(long __mask);

extern void __sret( void );

extern int __priority(pid_t __pid1, int __pri);

extern int __netdata(pid_t __pid1, unsigned __type );

extern void __yield(void);

extern int __sendfdmx(
            int                   __fd,
            unsigned              __smsg_xparts,
            unsigned              __rmsg_xparts,
            struct _mxfer_entry  *__smsg,
            struct _mxfer_entry  *__rmsg );


extern int __trigger(pid_t __pid);

extern int __semaphore(int id, int semaction);

#ifdef __cplusplus
};
#endif

#if defined __WATCOMC__ && !defined __WCC_KERNEL_H_INCLUDED
 #include <sys/wcc_kernel.h>
#endif

#else

#if !defined __KERNEL16_H_INCLUDED
 #include <sys/kernel16.h>
#endif

#endif

#endif
