/*-
 * <sys/kev.h> - kernel events
 */

#ifndef _SYS_KEV_H_INCLUDED
#define _SYS_KEV_H_INCLUDED

#ifndef __TYPES_H_INCLUDED
#include <sys/types.h>
#endif

/*
 *  |pccMMMMM|Mmmmmmmm|dddddddd|dddddddd|
 * 31                                   0
 *
 * p - Pointer    1 bit
 * c - Count      2 bits
 * M - Major code 6 bits
 * m - Minor code 7 bits
 * d - User data 16 bits
 *
 * If Major code == 3f then major code is in User data.
 */

struct _kev_code {
        unsigned long data    : 16,
                      minor   :  7,
                      major   :  6,
                      count   :  2,
                      pointer :  1;
};
        

struct kev {
        union {
                struct _kev_code _bits;
                ulong_t          _val;
        } kev_un;
#define kev_code kev_un._bits
#define kev_op   kev_un._val
        unsigned long kev_nsec;
};

#define EMAXCNT        3
#define EPOINTER       4
#define EISPTR(ecode)  ((ecode) & 0x80000000)
#define ECOUNT(ecode)  (((ecode) >> 29) & 0x3)
#define EOPCODE(ecode) (((ecode) >> 16) & 0x1fff)
#define EMAJOR(ecode)  (EOPCODE(ecode) >> 7)
#define EMINOR(ecode)  (EOPCODE(ecode) & 0x7f)
#define EDATA(ecode)   ((ecode) & 0xffff)
#define ECODE(code, dat16, count) \
             (((count)<<29)|((code)<<16)|((dat16)&0xffff))
#define EOPNO(m, l)     ((m)<<7 | (l))




/*-
 * Kev is a var-args function, but if you notice we want the first
 * argument in a register.   This can't be done with prototypes, as
 * the compiler will over-ride, or get picking about the argument
 * information, so we use the coy prototype instead.
 * The Kev[012] are just to keep the casts out of the source.
 */
extern int Kev();

#if defined __WATCOMC__ && defined __386__
#ifdef _OLD_KMON
#pragma aux Kev = "int 0xfb" parm caller [eax] modify nomemory exact [eax];
#else
#pragma aux _Kev = "int 0xfb" parm caller [eax] modify nomemory exact [eax];
#pragma aux Kev parm caller [eax];
#endif
#endif

#define Kev0(e)      Kev(e)
#define Kev1(e,f)    Kev(e, (ulong_t)f)
#define Kev2(e,f,g)  Kev(e, (ulong_t)f, (ulong_t)g)


struct _event_ker_proc_stat {
        nid_t           nid;
        long            pid;
        long            flags;
        long            blocked_on;
        nid_t           sid_nid;
        long            pid_group;
        char            state;
        char            priority;
        short int       sid;
        union {
                struct {
                        long    father;
                        long    son;
                        long    brother;
                        char    name[100];
                        } pidinfo;
                struct {
                        nid_t   remote_nid;
                        long    remote_pid;
                        long    local_pid;
                        } vidinfo;
                struct {
                        long int count;
                        } proxyinfo;
                } un;
        } ;

/* major codes */
#define _EVENT_MON  1
#define _EVENT_KER  2
#define _EVENT_USER 3

enum MonEvents {
        _EVENT_MON_TICK_WRAP   = EOPNO(_EVENT_MON, 1),
        _EVENT_MON_MISSED      = EOPNO(_EVENT_MON, 2),
        _EVENT_MON_SYN         = EOPNO(_EVENT_MON, 3)
};

enum KerEvents {
        _EVENT_KER_IRQ         =  EOPNO(_EVENT_KER, 1),
        _EVENT_KER_SEND        =  EOPNO(_EVENT_KER, 2),
        _EVENT_KER_RECEIVE     =  EOPNO(_EVENT_KER, 3),
        _EVENT_KER_REPLY       =  EOPNO(_EVENT_KER, 4),
        _EVENT_KER_CRECEIVE    =  EOPNO(_EVENT_KER, 5),
        _EVENT_KER_READMSG     =  EOPNO(_EVENT_KER, 6),
        _EVENT_KER_WRITEMSG    =  EOPNO(_EVENT_KER, 7),
        _EVENT_KER_RELAY       =  EOPNO(_EVENT_KER, 8),
        _EVENT_KER_SRET        =  EOPNO(_EVENT_KER, 9),
        _EVENT_KER_PRIORITY    =  EOPNO(_EVENT_KER, 10),
        _EVENT_KER_NETDATA     =  EOPNO(_EVENT_KER, 11),
        _EVENT_KER_YIELD       =  EOPNO(_EVENT_KER, 12),
        _EVENT_KER_SENDFD      =  EOPNO(_EVENT_KER, 13),
        _EVENT_KER_TRIGGER     =  EOPNO(_EVENT_KER, 14),
        _EVENT_KER_SPECIAL     =  EOPNO(_EVENT_KER, 15),
        _EVENT_KER_SIGNAL      =  EOPNO(_EVENT_KER, 16),
        _EVENT_KER_APM         =  EOPNO(_EVENT_KER, 17),
        _EVENT_KER_SEMAPHORE   =  EOPNO(_EVENT_KER, 18),

        _EVENT_KER_PROC_STAT   =  EOPNO(_EVENT_KER, 19),
        _EVENT_KER_DEATH       =  EOPNO(_EVENT_KER, 20),
        _EVENT_KER_PROXY       =  EOPNO(_EVENT_KER, 21),
        _EVENT_KER_ACTIVE      =  EOPNO(_EVENT_KER, 22),
        _EVENT_KER_READY       =  EOPNO(_EVENT_KER, 23),
        _EVENT_KER_IRQDONE     =  EOPNO(_EVENT_KER, 24),
        _EVENT_KER_USER_IRQ    =  EOPNO(_EVENT_KER, 25),
        _EVENT_KER_USER_EOI    =  EOPNO(_EVENT_KER, 26),
/*- fault, */
        _EVENT_KER_ENDFAULT    =  EOPNO(_EVENT_KER, 27),
/*- active, eip, cs:fault#*/
        _EVENT_KER_FAULT       =  EOPNO(_EVENT_KER, 28),
        _EVENT_KER_KILL        =  EOPNO(_EVENT_KER, 29),
/*- src,dest,nbytes*/
		_EVENT_KER_XFER        =  EOPNO(_EVENT_KER, 30),

        _EVENT_KER_NOP         =  EOPNO(_EVENT_KER, 31),

/*- kcall#, dest, errno */
        _EVENT_KER_ERROR       = EOPNO(_EVENT_KER, 32)
};


enum UserEvents {
	_EVENT_USER_TRON    = EOPNO(_EVENT_USER, 1), /*arg=addr,ret,arg0*/
	_EVENT_USER_TROFF   = EOPNO(_EVENT_USER, 2), /*arg=addr,ret,retval*/
	_EVENT_USER_TRFLUSH = EOPNO(_EVENT_USER, 3)  /*noarg*/
};



enum {
        Cycle8253 = 0,  /* cycles counted via 8253 */
        CycleP5   = 1,  /* cycles counted via Pentium(tm) counter */
};

#define KEV_MAGIC      'KEV0'
#define KEV_VERSION    1
#define KEV_ISMAG(m)  ((m) == KEV_MAGIC)

struct kev_header {
        ulong_t      magic;     
        ulong_t      version;
        ulong_t      dataoffs;          /* offset of samples */
        ulong_t      nsamples;          /* number of samples */
        ulong_t      nid;               /* node for this log */
        ulong_t      nprocs;            /* max number of processes */
        ulong_t      cpu;               /* cpu type */
        ulong_t      time_start[2];     /* utd, nsec. */
        ulong_t      time_stop[2];      /* utd, nsec. */
        ulong_t      clicks_start[2];
/* timer information: */
        uchar_t      cycle_type,        /* see enum above */
                     :8,:8,:8;
        ulong_t      cycle_arg; /* timer reload, P5 cpu period, ... */
        ulong_t      tick_per_sec;
};

#endif
