/*-
 * This file defines a number of structures of system information
 * gathering implemented in a variety of gathering programs.
 */
#ifndef sysinfo_h
#define sysinfo_h

#include <lmath.h>
#include <sys/stat.h>

#define SYS_NPRIO   32
#define SYS_NINTR   16


#define SYSI_NAME "/SysI"

struct sysinfo_directory {
#define SYSI_MAGIC ('S'<<24|'y'<<16|'s'<<8|'i')
	unsigned long sysi_magic;
	unsigned long diroffs;
	unsigned long ndirs;
};

struct sysinfo_direntry {
	unsigned long dirlen;
	unsigned long dirtype;
	unsigned long dataoffs;
	unsigned long datalen;
};

#define SIT_PRIO_LOAD   001
struct sysinfo_prioload {
	l64           priotimes[SYS_NPRIO];
};
#define SIT_INTR_LOAD   002
struct sysinfo_intrload {
	l64           intrtimes[SYS_NINTR];
};


typedef struct sysidir      SYSI_DIR;
typedef struct sysidirent   SYSI_DIRENT;

struct sysidir {
	int          isok;
	int          fd;
	struct stat  st;
	int          index;
	union {
		char        *map;
		struct sysinfo_directory *dir;
	}            u;
};

struct sysidirent {
	long         type;
	long         offset;
	long         len;
	void        *map;
};

extern int sysi_diropen(char *name, SYSI_DIR *dir);
extern int sysi_dirclose(SYSI_DIR *d);
extern int sysi_seekdir(SYSI_DIR *dir, int n);
extern int sysi_readdir(SYSI_DIR *dir, SYSI_DIRENT *de);


#endif
