
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/kernel.h>
#include <sys/mman.h>
#include <sys/psinfo.h>
#include <sys/proc_msg.h>
#include <sys/osinfo.h>
#include <sys/inline.h>
#include <sys/proxy.h>
#include <sys/irqinfo.h>
#include <sys/sched.h>
#include "monitor.h"
#include "kertest.h"

int            mprio = 12;
struct pidin  *pidstat;
unsigned       pidmask;
unsigned       nproc;
int            tickleproxy;

void ps_fill();

initmem()
{
	struct _osinfo o;
	int     n;

	if (qnx_osinfo(0, &o) == -1 ||
	    (pidstat = malloc((nproc=o.pidmask+1)*sizeof *pidstat)) == 0) {
		return -1;
	}
	pidmask = o.pidmask;
	for (n=0; n < nproc; n++) {
		pidstat[n].st = PROC_EMPTY;
	}
	free(malloc(8000));  /* reserve for stdio et al */
	return 0;
}


EvAp_done(int ecode)
{
	exit(0);
}

done()
{
	return -1; /*signals an exit*/
}

/* look at _event_ker_proc_stat in /usr/include/sys/kev.h for full info
   on the event that just occurred. */

printproc(char *msg, struct pidin *p)
{
	printf("%s:%u ", msg, p->proc.pid );

	/* dump pidin structure (_event_ker_proc_stat) */ 
	printf("nid=%d, fl=0x%x, bl=%ld,",
				p->proc.nid, p->proc.flags, p->proc.blocked_on);
	printf("state=%d, prio=%d, sid=%d,",
				p->proc.state, p->proc.priority, p->proc.sid);

	switch (p->proc.flags & (_PPF_VID|_PPF_MID|_PPF_VMID)) {
	case 0:
		printf("%s", p->proc.un.pidinfo.name);
		break;
	case _PPF_VID:
		printf("vc from %u to %lu:%u",
			p->proc.un.vidinfo.local_pid,
			p->proc.un.vidinfo.remote_nid,
			p->proc.un.vidinfo.remote_pid);
		break;
	case _PPF_MID:
		printf("proxy to %u", p->proc.blocked_on);
		break;
	case _PPF_VMID:
		printf("vmid");
		break;
	}
	printf("\n");
	fflush(stdout);
}


main(int argc, char *argv[])
{
	int n;
	ulong_t          irqmask = 0;
	int              type = _MON_AUTO;

	while ((n = getopt(argc, argv, "p:")) != -1) {
		switch(n) {
		case 'p':
			mprio = strtol(optarg, 0, 0);
			setprio(0, mprio);
			break;
		default:
			exit( EXIT_FAILURE );
		}
	}

	if ((tickleproxy=qnx_proxy_attach(0,0,0,-1)) == -1 ||
	    qnx_hint_attach(-1, tickleme, _ss()) == -1) {
		exit(1);
	}
	if (initmem() == -1) {
		exit(1);
	}
	moninstall(type, 0);
	monsignal(SIGINT, done);
	monsignal(SIGTERM, done);
	monsignal(SIGQUIT, done);
	monsignal(SIGKILL, done);

	_evstart(getpid());
	ps_fill();
	time(0);
	qnx_kev_enable(-1, irqmask);
	while (1) {
		int  pid;
		if ((pid=Receivemx(0,0,0)) == tickleproxy) {
			for (pid=0; pid < nproc; pid++) {
				struct pidin p;
				_disable();
				switch (pidstat[pid].st) {
				case PROC_CREATED:
					p = pidstat[pid];
					pidstat[pid].st = PROC_MAPPED;
					_enable();
					printproc( "NEW", &p);
					break;
				case PROC_DEAD:
					pidstat[pid].st = PROC_EMPTY;
					_enable();
					printproc( "DEAD", &p );
					break;
				default:
					enable();
					break;
				}
			}
		}
	}
	monshutdown();
}

