/*-
 * installing an event reader.
 */

#include <errno.h>
#include <i86.h>
#include <signal.h>
#include <sys/seginfo.h>
#include <sys/inline.h>
#include <sys/osinfo.h>
#include <sys/psinfo.h>
#include "monitor.h"
#include "monitor-private.h"

struct _osinfo       monosdata;
struct _timesel far *montimep;
unsigned             monds;
struct KernelEvent   monevent;


static unsigned moncs;

#pragma pack (1)
struct _i86gate {
	unsigned short  offset;
	unsigned short  selector;
	unsigned char   word_count;
	unsigned char   flags;
	unsigned short  offs2;
};
#pragma pack()


static int psf = _PPF_IMMORTAL|
                 _PPF_SIGMASKALL|
                 _PPF_PRIORITY_REC|
                 _PPF_PRIORITY_FLOAT;

static sigset_t  block;   /* signals which are not allowed */
static sigset_t  caught;  /* signals to stop on */
static sigset_t  handled; /* signals to invoke a handler on */ 

static int (*sighandlers[_SIGMAX])();

static int    trap_installed;
static struct _i86gate original_trap;

static
setvec(int vec, struct _i86gate *new, struct _i86gate *old, int *flag)
{
	struct _i86gate far *idt = MK_FP(0x10, 0);

	_disable();
	if (old) {
		*old = idt[vec];
	}
	idt[vec] = *new;
	if (flag) {
		*flag = old != 0;
	}
	_enable();
}

void
monshutdown()
{
	qnx_kev_disable(); /*STOP PROC RECORDING*/
	if (trap_installed) {
		setvec(MONTRAP, &original_trap, 0, &trap_installed);
	}
}

static
sigcatch(int sig)
{
	if (sigismember(&handled, sig)) {
		if (sighandlers[sig] == 0 ||
		   (sighandlers[sig])(sig) < 0) {
			monshutdown();
			EvAp_done(-sig);
		}
	} else if (sigismember(&caught, sig)) {
		monshutdown();
		EvAp_done(-sig);
	} else {
		/*Oops; just ignore it?!? */
	}
}

int
monsignal(int signo, int (*f)())
{
	if (signo < _SIGMIN || signo > _SIGMAX) {
		return EINVAL;
	}
	sighandlers[signo] = f;
	if (sigismember(&handled, signo) == 0) {
		signal(signo, sigcatch);
		sigaddset(&handled, signo);
	}
	if (sigismember(&block, signo)) {
		sigdelset(&block, signo);
		sigprocmask(SIG_SETMASK, &block, 0);
	}
	return 0;
}

char *
moninstall(int type, sigset_t *stop)
{
	void (*recorder)();
	struct _i86gate handler;
	int             i;

	qnx_osinfo(0L, &monosdata);
	montimep    = MK_FP(monosdata.timesel, 0);
	qnx_pflags(psf, psf, 0, 0);
	sigemptyset(&block);
	sigemptyset(&caught);
	sigemptyset(&handled);
	for (i=_SIGMIN; i <= _SIGMAX; i++) {
		if (stop && sigismember(stop, i)) {
			signal(i, sigcatch);
			sigaddset(&caught, i);
		} else {
			sigaddset(&block, i);
		}
	}
	sigprocmask(SIG_SETMASK, &block, 0);

	switch (type) {
	case _MON_AUTO:
	default:
		if (monosdata.cpu >= 586) {
			recorder = _rdtsc_swi;
		} else {
			recorder = _8254_swi;
		}
		break;
	case _MON_RDTSC:
		recorder = _rdtsc_swi;
		break;
	case _MON_8254:
		recorder = _8254_swi;
		break;
	}

	if ((monds = qnx_segment_put(0, _ds(), _PMF_DATA_RW|_PMF_GLOBAL|_PMF_GLOBAL_OWN)) != -1)
		moncs = qnx_segment_put(0, _cs(), _PMF_CODE_RX|_PMF_GLOBAL|_PMF_GLOBAL_OWN);

	if (moncs == -1  ||  monds == -1) {
		return "could not install global selectors";
	}
	if (qnx_segment_priv(moncs, 0) == -1) { /* Make conforming code segment.*/
		return "Could not set privity";
	}
	handler.offset   = (unsigned)recorder & 0xffff;
	handler.selector = moncs&~03;
	handler.word_count = 0;
	handler.flags = 0xee;
	handler.offs2    = ((unsigned)recorder>>16) & 0xffff;
	setvec(MONTRAP, &handler, &original_trap, &trap_installed);
	return 0;
}
