
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sysinfo.h>


int
sysi_diropen(char *name, SYSI_DIR *dir)
{

	if (name == 0)
		name = SYSI_NAME;
	dir->isok = 0;
	if ((dir->fd=shm_open(name, O_RDONLY, 0)) != -1) {
		if (fstat(dir->fd, &dir->st) != -1) {
			if ((dir->u.map=mmap(0, dir->st.st_size, PROT_READ, MAP_SHARED, dir->fd, 0)) != MAP_FAILED) {
				if (dir->u.dir->sysi_magic == SYSI_MAGIC) {
					dir->isok = 1;
					dir->index  = 0;
					return 0;
				}
				munmap(dir->u.map, dir->st.st_size);
			}
		}
		close(dir->fd);
	}
	return -1;
}

int
sysi_dirclose(SYSI_DIR *d)
{
	if (d->isok) {
		munmap(d->u.map, d->st.st_size);
		close(d->fd);
		d->isok = 0;
		return 0;
	}
	return -1;
}

sysi_seekdir(SYSI_DIR *dir, int n)
{
	if (n<dir->index) {
		dir->index = n;
		return 0;
	}
	return -1;
}

sysi_readdir(SYSI_DIR *dir, SYSI_DIRENT *de)
{

	if (dir->index < dir->u.dir->ndirs) {
		union {
			char *p;
			struct sysinfo_direntry *dp;
		} u;
		int i;
		for (u.p=dir->u.map+dir->u.dir->diroffs, i=0; i < dir->index; i++) {
			u.p += u.dp->dirlen;
		}
		de->type = u.dp->dirtype;
		de->len  = u.dp->datalen;
		de->offset = u.dp->dataoffs;
		de->map    = dir->u.map + u.dp->dataoffs;
		dir->index++;
		return 0;
	}
	return -1;
}
