
#ifndef kevio_h
#define kevio_h 1

#ifndef _STDIO_H_INCLUDED
#include <stdio.h>
#endif

#ifndef __SYS_EVENT_H_INCLUDED
#include <sys/kev.h>
#endif

#ifndef lmath_h_included
#include "lmath.h"
#endif

#if defined __cplusplus
extern "C" {
#endif

typedef struct _KFilt KFilt;
typedef struct _KFILE KFILE;
typedef struct _KRec KRec;
typedef struct _Proc Proc;
typedef struct _KSTAT KSTAT;

struct _KRec {
	union {
		struct {
			unsigned  kmin:7,
				  kmaj:6;
		} kpart;
		ulong_t     kval;
	} kcode;
#define kr_major kcode.kpart.kmaj
#define kr_minor kcode.kpart.kmin
#define kr_opcode kcode.kval
	l64            kr_ticks;
	ulong_t        kr_oprnd;
	ulong_t        kr_nwords;
	ulong_t        kr_data[1];
};

struct _KSTAT {
	ulong_t      cpu;
	ulong_t      samples;
	ulong_t      tick_sec;
};

struct _Proc   {
	pid_t       pid;
	char       *name;	/* normal name */
	char       *sname;	/* system name */
	char        state;
	pid_t       lpid;    /* link pid (for proxies) */
};

#define KevMkCode(maj, min) EOPNO(maj,min)

/* maintain the KFILE */
extern KFILE  *KevOpen(char *);
extern KFILE  *KevFopen(FILE *);
extern void    KevClose(KFILE *);
extern void    KevRewind(KFILE *);
extern KFilt  *KevAddFilt(KFILE *, int (*f)(), void *arg);
extern void    KevDelFilt(KFILE *, KFilt *);

/* get event records */
extern KRec   *KevNext(KFILE *);
extern KRec   *KevThis(KFILE *);
extern KRec   *KevDup(KRec *);
extern KRec   *KevDel(KRec *);

/* access state information */
extern int     KevGetStat(KFILE *, KSTAT *);
extern Proc   *KevGetProc(KFILE *, unsigned);
extern int     GetProcName(KFILE *fp, unsigned pid, char *buf, unsigned blen);

/* printing functions */
extern int     dumpev(KRec *, FILE *);
extern int     print_time(KRec *, FILE *);
extern int     print_ipc(KRec *e, int type, char *src, char *dest, FILE *fp);
extern int     print_pstat(KFILE *fp, struct _event_ker_proc_stat *ps, FILE *outf);
extern int     print_kerevent(KFILE *fp, KRec *e, FILE *outf);

#if defined __cplusplus
}
class KEV {
	KFILE      *kp;
public:
	KEV(char *fn)       { kp = KevOpen(fn); }
	KEV(FILE *fp)       { kp = KevOpen(fn); }
	~KEV()              { KevClose(kp); kp = 0; }
	int get(KRec **p)   { return kp ? (*p=KevNext)!=0 : 0; }
	KRec *cur()         { return kp ? KevThis(kp) : 0; }
	void  rewind()      { if (kp) KevRewind(kp); }
};
#endif

extern char *nlookup(ulong_t a);
#endif
