/*-
 * Inline functions and definitions for 64bit math:
 *
 * l64stat_t ul_add(dst, src);
 * -add src value to dst. return 1 if carry.
 *
 * l64stat_t ul_sub(dst, src);
 * -subtract src from dst. return 1 if borrow.
 *
 * l64stat_t ul_addu(dst, src);
 * -add src long to dst value. return 1 if carry.
 *
 * l64stat_t ul_subu(dst, src);
 * -subtract src long from dst. return 1 if borrow.
 *
 */

#ifndef lmath_h_included
#define lmath_h_included

#ifndef __TYPES_H_INCLUDED
#include <sys/types.h>
#endif


#if defined __cpluspluse
extern "C" {
#endif

typedef ulong_t  l64[2];
typedef signed char l64stat_t;


extern l64stat_t  ul_add(l64 dst, l64 src);
extern l64stat_t  ul_sub(l64 dst, l64 src);
extern l64stat_t  ul_mul(l64 dst, l64 src);
extern l64stat_t  ul_div(l64 dst, l64 src);
extern l64stat_t  ul_mod(l64 dst, l64 src);
extern l64stat_t  ul_divmod(l64 mod, l64 dst, l64 src);

extern l64stat_t  ul_addu(l64 dst, ulong_t val);
extern l64stat_t  ul_subu(l64 dst, ulong_t val);
extern l64stat_t  ul_mulu(l64 dst, ulong_t val);
extern l64stat_t  ul_divu(l64 dst, ulong_t val);
extern l64stat_t  ul_modu(l64 dst, ulong_t val);
extern l64stat_t  ul_divmodu(l64 mod, l64 dst, ulong_t val);


extern l64stat_t  long2ul(l64 dst, long val);
extern l64stat_t  ulong2ul(l64 dst, ulong_t val);
extern l64stat_t  ul2long(long *dst, l64 src);
extern l64stat_t  ul2ulong(ulong_t *dst, l64 src);

extern void       ul_copy(l64 dst, l64 src);

extern l64stat_t  ul_cmp(l64 arg1, l64 arg2);
/*-
 * comparison operators:
 */
#define ul_gt(x,y) (ul_cmp(x,y) > 0)
#define ul_ge(x,y) (ul_cmp(x,y) >= 0)
#define ul_eq(x,y) (ul_cmp(x,y) == 0)

#define ul_lt(x,y) ul_gt(y,x)
#define ul_le(x,y) ul_ge(y,x)
#define ul_ne(x,y) !ul_eq(x,y)



#define ul_make(d, h, l) ((d)[0] = (l), (d)[1] = (h))
#define ul_copy(a,b)   ((a)[0] = (b)[0], (a)[1] = (b)[1])


#pragma aux ul_add = \
	"mov	ebx, [edx]" \
	"add	[eax], ebx" \
	"mov	ebx, [edx+4]" \
	"adc	[eax+4], ebx" \
	"setc	al" \
	parm [eax] [edx] modify exact [ebx al] value [al];
#pragma aux ul_sub = \
	"mov	ebx, [edx]" \
	"sub	[eax], ebx" \
	"mov	ebx, [edx+4]" \
	"sbb	[eax+4], ebx" \
	"setc	al" \
	parm [eax] [edx] modify exact [al] value [al];

#pragma aux ul_mul = \
	"fild	qword ptr [eax]" \
	"fild	qword ptr [edx]" \
	"fmul" \
	"fistp	qword ptr [eax]" \
	parm [eax] [edx] modify exact [8087 al] value [al];

#pragma aux ul_addu = \
	"add	[eax], edx" \
	"adc	[eax+4], 0" \
	"setc	al" \
	parm [eax] [edx] modify exact [al] value [al];
#pragma aux ul_subu = \
	"sub	[eax], edx" \
	"sbb	[eax+4], 0" \
	"setc	al" \
	parm [eax] [edx] modify exact [al] value [al];

/*-
 * notice the arguments for this forces the long value onto the stack.
 * The x87 can't access registers directly.
 */
#pragma aux ul_mulu = \
	"fild	qword ptr [eax]" \
	"fild	dword ptr [esp]" \
	"fmul" \
	"fistp qword ptr [eax]" \
	parm [eax] modify exact [8087 al] value [al];


#pragma aux ul_cmp = \
	"mov	ebx, [eax+4]" \
	"cmp	ebx, [edx+4]" \
	"ja     bigger" \
	"mov	ebx, [eax]" \
	"cmp	ebx, [edx]" \
	"ja	bigger" \
	"sete	al" \
	"dec	al" \
	"jmp    done" \
	"bigger: mov al,1" \
	"done:" \
	parm [eax] [edx] modify [ebx eax] value [al];



/*-
 * conversion functions
 */

#pragma aux long2ul = \
	"cdq" \
	"mov    [ebx], eax" \
	"mov	[ebx+4], edx" \
	"setc	al" \
	parm [ebx] [eax] modify exact [eax edx] value [al];

#pragma aux ulong2ul = \
	"mov	[edx], eax" \
	"xor	eax, eax" \
	"mov	[edx+4], eax" \
	parm [edx] [eax] modify exact [eax] value [al];

#pragma aux ul2long = \
	"push	dword ptr [edx]" \
	"pop	dword ptr [eax]" \
	"mov	eax, [edx+4]" \
	"and	eax, eax" \
	"setc	al" \
	parm [eax] [edx] modify exact [eax] value [eax];

#pragma aux ul2ulong = \
	"push	dword ptr [edx]" \
	"pop	dword ptr [eax]" \
	"mov	eax, [edx+4]" \
	"and	eax, eax" \
	"setc	al" \
	parm [eax] [edx] modify exact [eax] value [eax];






#if defined __cplusplus
};

class vast {
private:
	l64	v;
public:
	vast(ulong_t n)			{ ul_make(v, 0, n); }
	vast(ulong_t a, ulong_t b)	{ ul_make(v, a, b); }
	vast(vast &vn)			{ ul_copy(v, vn); }
	~vast() {}
};


#endif

#endif
