#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

#define MAX_FILES 5
static FILE *msg_fp[MAX_FILES], *ind_fp[MAX_FILES];

#define CACHE
#ifdef CACHE
struct _cache {
	struct _cache *next, *prev;
	short type, subtype;
	char *str;
} *head;

void
cache(int type, int subtype, char *str) {
	struct _cache *h;

	for (h = head; h; h = h->next) {
		if (h->type == type && h->subtype == subtype)
			return;
	}
	if (!(h = malloc(sizeof(struct _cache))))
		return;
	h->type = type;
	h->subtype = subtype;
	h->str = strdup(str);
	h->prev = 0;
	h->next = head;
	if (head)
		head->prev = h;
	head = h;
}

char *
find_cache(int type, int subtype) {
	struct _cache *h;

	/* keep an MRU cache */
	for (h = head; h; h = h->next) {
		if (h->type == type &&
		    (h->subtype == subtype || h->subtype == -1)) {
			if (h != head) {
				/* remove element */
				h->prev->next = h->next;
				if (h->next)
					h->next->prev = h->prev;
				/* insert element at head */
				h->prev = 0;
				h->next = head;
				if (head)
					head->prev = h;
				head = h;
			}
			return h->str;
		}
	}
	return 0;
}
#endif

int
msg_open(char *file) {
	char buf[PATH_MAX];
	int i;

	if (file == 0)
		file = "/usr/lib/msgs/qssl";

	for (i = 0; i < MAX_FILES; i++)
		if (msg_fp[i] == 0)
			break;
	if (i == MAX_FILES) {
		errno = ENOMEM;
		return -1;
	}

	if (!(msg_fp[i] = fopen(file, "r")))
		return -1;
	sprintf(buf, "%s.ind", file);
	if (!(ind_fp[i] = fopen(buf, "r"))) {
		fclose(msg_fp[i]); msg_fp[i] = 0;
		return -1;
	}
	return 0;
}

void
msg_close(void) {
	int i;

	for (i = 0; i < MAX_FILES; i++) {
		if (msg_fp[i]) fclose(msg_fp[i]);
		if (ind_fp[i]) fclose(ind_fp[i]);
		msg_fp[i] = ind_fp[i] = 0;
	}
}

char *
msg_lookup(void *msg, int len) {
	static char hextype[7];
	static char buf[200];
	char *p;
	msg_t type;
	short subtype;
	off_t off;
	int i;

	if (len < sizeof(msg_t))
		return 0;
	type = *(msg_t *)msg;
	if (len >= (sizeof(msg_t)+sizeof(short)))
		subtype = *(short *)((char *)msg+sizeof(msg_t));
	else
		subtype = -1;
#ifdef CACHE
	if (p = find_cache(type, subtype))
		return p;
#endif
	sprintf(hextype, "0x%04x", type);
	for (i = 0; i < MAX_FILES; i++) {
		if (msg_fp[i] == 0 ||
		    fseek(ind_fp[i], type*sizeof(off_t), SEEK_SET) != 0 ||
		    fread(&off, sizeof(off_t), 1, ind_fp[i]) != 1 ||
		    fseek(msg_fp[i], off-1, SEEK_SET) != 0)
			continue;
		if (getc(msg_fp[i]) != '\n')
			return 0;
		while (fgets(buf, sizeof buf, msg_fp[i]) &&
				strtol(buf, 0, 0) == type) {
			char *p;

			if (*buf == '#')
				continue;
			if (p = strchr(buf, ','))
				p++;
			else
				continue;
			if (*p == '*' || strtol(p, 0, 0) == subtype) {
#ifdef CACHE
				cache(type, *p == '*' ? -1 : subtype, buf);
#endif
				return buf;
			}
		}
	}
	return hextype;
}
