/*-
 * proc.c: maintain a table containing process information.
 */

#include <string.h>
#include <stdlib.h>
#include <sys/psinfo.h>
#include "kevio.h"
#include "private.h"

#ifndef debug
#define debug 0
#endif

#define RELEASE(P)	if (P) { free(P); P=0; }


#define PID_SLOT(fp, pid)  ((fp)->proctab+PID_INDEX(fp, pid))

static IProc *
ChkPid(fp, pid)
KFILE          *fp;
unsigned        pid;
{
	IProc    *thisp = PID_SLOT(fp, pid);
//	if (thisp && thisp->p.pid != pid) {
	if (thisp && thisp->valid && thisp->p.pid != pid) {
		if (debug) {
			fprintf(stderr, "Mismatched: wanted pid %u, got %u\n",
				pid, thisp->p.pid);
		}
	}
	return thisp;
}

__proc_init(KFILE *fp)
{
	IProc     *proctab;
	unsigned   i;
	if ((proctab = calloc(fp->hdr.nprocs+2, sizeof *proctab)) == 0)
		return -1;
	fp->proctab = proctab;
	for (i=1; i && i < fp->hdr.nprocs; i<<=1);
	fp->pidhash = i-1;
	return 0;
}

int
__proc_update(KFILE *fp, KRec *kr)
{
	struct _event_ker_proc_stat *ps;
	IProc                       *procp;
	char                        *s;
	char                         buf[128];
	if (fp->proctab == 0)
		return 0;

	ps = (struct _event_ker_proc_stat *)kr->kr_data;
	procp = ChkPid(fp, ps->pid);
	if (procp->valid) {
		if (debug) {
			fprintf(stderr, " proc (%u) redefined, not dead!\n",
				ps->pid);
		}
		if (procp->p.name && debug) {
				fprintf(stderr, " freeing proc (%d:%s)\n", procp->p.pid, procp->p.sname);
		}
		RELEASE(procp->p.name);
		RELEASE(procp->p.sname);
		memset(&procp->p, 0, sizeof procp->p);
	}
	procp->valid = 1;
	procp->p.pid = ps->pid;
	procp->p.state = ps->state;
	procp->p.lpid = 0;
	if (ps->flags & _PPF_VMID) {
		sprintf(buf, "%d(vmid)", procp->p.pid);
		procp->p.sname = strdup(buf);
		procp->p.name = strdup(buf);
	} else if (ps->flags & _PPF_VID) {
		sprintf(buf, "%d(vid)", procp->p.pid);
		procp->p.sname = strdup(buf);
		procp->p.name = strdup(buf);
	} else if (ps->flags & _PPF_MID) {
		sprintf(buf, "%d(proxy)", procp->p.pid);
		procp->p.lpid = ps->blocked_on;
		procp->p.sname = strdup(buf);
		procp->p.name = strdup(buf);
	} else {
		s = ps->un.pidinfo.name;
		procp->p.sname = strdup(ps->un.pidinfo.name);
		s = basename(ps->un.pidinfo.name);
		sprintf(buf, "%d(%s)", procp->p.pid, s);
		procp->p.name = strdup(buf);
	}
	return 0;
}

int
__proc_delete(KFILE *kfp, unsigned pid)
{
	IProc                        *procp;
	if (kfp->proctab == 0)
		return 0;
	procp = ChkPid(kfp, pid);
	if (debug && procp->valid)
		fprintf(stderr, "reusing slot %u\n", PID_INDEX(kfp, pid));
	procp->valid = 0;
	RELEASE(procp->p.name);
	RELEASE(procp->p.sname);
	return 0;
}

Proc *
KevGetProc(KFILE *kfp, unsigned pid)
{
	IProc      *procp;
	if (kfp->proctab == 0)
		return 0;

	procp = ChkPid(kfp, pid);
	if (!procp->valid) {
		if (debug) {
			if (pid > 0x10000) {
				fprintf(stderr, "(impossible pid %u)\n", pid);
				return 0;
			}
			fprintf(stderr, "warning: lookup (%u) = dead pid\n", pid);
		}
		return 0;
	}
	return &procp->p;
}
