
/*-
 * KevIO: io package for Kev handling
 */
#include <string.h>
#include <stdio.h>
#include "kevio.h"
#include "private.h"


KRec *
kev_read(KFILE *kfp)
{
	struct kev     krec;
	ulong_t        imm[EMAXCNT];
	ulong_t        elen = 0;
	int            i;
	KRec          *kp;
	FILE          *fp = kfp->fp;

	for (;;) {
	    if (fread(&krec, sizeof krec, 1, fp) != 1)
		return 0;
	    if (krec.kev_op)
		break;
	    fseek(fp, -sizeof krec.kev_op, SEEK_CUR);	// backup half a krec
	}
	if (fread(&imm, sizeof *imm, krec.kev_code.count, fp) !=
		krec.kev_code.count) {
		return 0;
	}
	if (krec.kev_code.pointer) {
		if (fread(&elen, sizeof elen, 1, fp) != 1) {
			return 0;
		}
	}
	if ((kp=malloc(sizeof *kp +
		sizeof *kp->kr_data * (elen+krec.kev_code.count-1))) == 0) {
		return 0;
	}
	for (i=0; i < krec.kev_code.count; i++) {
		kp->kr_data[i] = imm[i];
	}
	if (fread(kp->kr_data+i, sizeof kp->kr_data[i], elen, fp) != elen) {
		return 0;
	}
	kp->kr_nwords = krec.kev_code.count + elen;
	kp->kr_oprnd  = krec.kev_code.data;
	kp->kr_opcode = 0;	/* clear union */
	kp->kr_major  = krec.kev_code.major;
	kp->kr_minor  = krec.kev_code.minor;
	switch (kp->kr_opcode) {
	case _EVENT_MON_TICK_WRAP:
		kfp->ktick = krec.kev_nsec;
		krec.kev_nsec = 0;
		break;
	case _EVENT_KER_PROC_STAT:
		proc_update(kfp, kp);
		break;
	case _EVENT_KER_DEATH:
		proc_delete(kfp, kp->kr_oprnd);
		break;
	}
	ul_make(kp->kr_ticks, kfp->ktick, krec.kev_nsec);
	return kp;
}

KRec *
KevNext(KFILE *kp)
{
	KRec      *k;
	int        nrec=1;
	if (kp->currec)
		free(kp->currec);
	if ((k=kev_read(kp)) == 0) {
		return 0;
	}
	if (kp->filter) {
		while (1) {
			int        res=0;
			KFilt     *f=kp->filter;
			while (f && !(res=f->filt(k,f->arg)))
				f=f->next;
			if (res)
				break;
			free(k);
			if ((k=kev_read(kp)) == 0)
				break;
			nrec++;
		}
	}
	kp->recno += nrec;
	return kp->currec = k;
}
