/*-
 * chktime: check that time in the log is moving forward.
 *      This should no longer be a problem, however it has been in the past.
 */

#include <libc.h>
#include "kevio.h"


/*-
 * Q routines to store the last n samples.
 */

typedef struct Q Q;

struct Q {
	unsigned	nelem;
	unsigned	nsiz;
	unsigned	head, tail;
	char           *ch;
};

Q *
mk_q(unsigned nelem, unsigned nsiz)
{
	Q *q=malloc(sizeof *q);
	q->nelem = nelem;
	q->nsiz =  nsiz;
	q->ch = malloc(nelem*nsiz);
	q->head = q->tail = 0;
	return q;
}
del_q(Q *q)
{
	if (q->ch) free(q->ch);
	free(q);
}

Qput(Q *q, void *p)
{
	unsigned h = q->head +1;
	if (h == q->nelem)
		h = 0;
	if (h == q->tail) {
		if (++q->tail == q->nelem)
			q->tail = 0;
	}
	memcpy(q->ch+q->head*q->nsiz, p, q->nsiz);
	q->head = h;
}

int
Qget(Q *q, void *p)
{
	unsigned t = q->tail+1;
	if (q->tail == q->head)
		return -1;
	if (t == q->nelem)
		t = 0;
	memcpy(p, q->ch+q->tail*q->nsiz, q->nsiz);
	q->tail = t;
	return 0;
}

int
Qpop(Q *q, void *p)
{
	unsigned t = q->head-1;
	if (q->head == q->tail)
		return -1;
	if (t > q->nelem) {
		t=q->nelem-1;
	}
	memcpy(p, q->ch+t*q->nsiz, q->nsiz);
	q->head = t;
	return 0;
}


void *
Qlast(Q *q)
{
	unsigned n=q->head-1;
	if (q->head == q->tail)
		return 0;
	if (n > q->nelem)	/* unsigned -wrap at 0*/
		n = q->nelem-1;
	return q->ch+n*q->nsiz;
}



chktimes(FILE *f)
{
	KFILE      *k = KevFopen(f);
	KRec       *p;
	struct last {
		l64          time;
		ulong_t      type;
	}           l, *lp;
	ulong_t     recno = 0;
	Q         *q=mk_q(6, sizeof l);


	ul_make(l.time, 0, 0);
	l.type = 0;
	if (k == 0) {
		fprintf(stderr, "not a sample file\n");
		return -1;
	}
	while (p=KevNext(k)) {
		if ((lp=Qlast(q)) && ul_lt(p->kr_ticks, lp->time)) {
			printf("%ld\t%04.4x\t", recno, p->kr_opcode);
			print_time(p, stdout);
			putchar('\n');
			while (Qget(q, &l)!=-1) {
				printf("-\t%04.4x\t%01x.%08.8x\n",
					l.type, l.time[1], l.time[0]);
			}
		} else {
			ul_copy(l.time, p->kr_ticks);
			l.type = p->kr_opcode;
			Qput(q, &l);
		}
		recno++;
	}
	del_q(q);
	return 0;
}

main(int argc, char **argv)
{
	int            i;
	while ((i=getopt(argc, argv, "")) != -1)
		;
	if (optind == argc) {
		chktimes(stdin);
	} else for (i=optind; i < argc; i++) {
		FILE          *f;
		if (f=fopen(argv[i], "r")) {
			printf("%s:\n", argv[i]);
			chktimes(f);
			fclose(f);
		} else {
			fprintf(stderr,"%s: unable to open '%s'\n",
				argv[0], argv[i]);
		}
	}
	return 0;
}
