
/*
 *	interrupts.c
 *
 *	QNX
 *
 *	(C) Copyright 1991, 1992, 1996 by Robert Krten, all rights reserved.
 *
 *	This module is responsible for monitoring interrupt rates
 *
 *	1991 09 21	R. Krten		created (states.c)
 *	1996 08 17	R. Krten		stole from states.c, modified
*/

#include "sysmon.h"
#include <sys/proc_msg.h>
#include <sys/irqinfo.h>

#define PINDEX(pid)		((pid) & 0x00ff)

extern	char *progname;					/* main.c */
extern	char *version;					/* version.c */

static	void	interruptMonitor (void);
static	void	totalinterrupts (void);

extern	int opta;
extern	int optims;
extern	struct _psinfo psData;
extern	char *lines;

//	irqNames contains the default names for the IRQs

char	irqNames [16][MaxIRQNameLen + 1] =
{
	{"Timr"},	// IRQ  0
	{"Keyb"},	// IRQ  1
	{"Casc"},	// IRQ  2
	{"    "},	// IRQ  3
	{"    "},	// IRQ  4
	{"    "},	// IRQ  5
	{"    "},	// IRQ  6
	{"    "},	// IRQ  7
	{"RTC "},	// IRQ  8
	{"    "},	// IRQ  9
	{"    "},	// IRQ 10
	{"    "},	// IRQ 11
	{"    "},	// IRQ 12
	{"Copr"},	// IRQ 13
	{"    "},	// IRQ 14
	{"    "}	// IRQ 15
};

int log10Table [] = {
1,		1,		1,		1,		2,		3,		3,		5,		6,		7,
10,		12,		15,		19,		25,		31,		39,		50,		63,		79,
100,	125,	158,	199,	251,	316,	398,	501,	630,	794,
1000,	1258,	1584,	1995,	2511,	3162,	3981,	5011,	6309,	7943,
10000,	12589,	15848,	19952,	25118,	31622,	39810,	50118,	63095,	79432,
100000,	125892,	158489,	199526,	251188,	316227,	398107,	501187,	630957,	794328,
1000000,1258925,1584893,1995262,2511886,3162277,3981071,5011872,6309573,7943282
};

static	struct _proc_counter_reply1 cnt [2];

void
doINTERRUPTS ()
{
	int		i;

	if (getcounts (cnt) == -1) {		// prime, and verify
		fatal ("your machine doesn't support kernel events");
	}
	getIRQNames ();

	term_cur (1, 0);
	term_clear (TERM_CLS_EOS);
	term_printf (2, 21, opta, "CPU Interrupts (Log scale)");
	term_printf (4,  0, opta, "Ŀ");
	term_printf (5,  0, opta, " Vector   Counts      10        100        1k        10k       100k      1M");
	term_printf (6,  0, opta, "Ĵ");
	for (i = 0; i < 16; i++) {
		term_printf (7 + i,  0, opta, " %2d %4.4s  .......                                                          ", i, irqNames [i]);
	}
	term_printf (23, 0, opta, "");
	term_flush ();
	
	for (;;) {
		interruptMonitor ();
		delay (optims);

		if (dev_ischars (0) && getc (stdin) == 'q') {
			break;
		}
	}
}

static void
interruptMonitor ()
{
	static int oddeven = 1;
	int		i;
	unsigned count;
	char	fmt [BUFSIZ];
	
	getcounts (cnt + oddeven);			// get the other buffer filled
	for (i = 0; i < 16; i++) {
		term_printf (i + 7, 12, opta, "%7u  ", count = cnt [oddeven].intr [i] - cnt [!oddeven].intr [i]);
		term_clear (TERM_CLS_EOL);
		if (count) {
			count = convertToLog (count);
			if (count > 7) {
				count -= 7;
			} else {
				count = 1;
			}
			sprintf (fmt, "%%%d.%ds", count, count);
			term_printf (7 + i, 22, opta, fmt, lines);
		}
		term_printf (7 + i, 78, opta, "");
	}
	term_flush ();
	oddeven = !oddeven;
}

getcounts (c)
struct	_proc_counter_reply1	*c;
{
	struct	_proc_counters	r;

	memset (&r, 0, sizeof (r));
	r.type = _PROC_COUNTERS;
	r.subtype = _PCOUNT_GETALL;
	return (Send (PROC_PID, &r, c, sizeof r, sizeof *c) == -1 || c->status != EOK)
		 ? -1 : 0;
}

convertToLog (c)
unsigned c;
{
	int		i;

	for (i = 0; i < (sizeof (log10Table) / sizeof (int)); i++) {
		if (c <= log10Table [i]) {
			return (i);
		}
	}
	return (i);
}

static int last_irqid;
static struct _osinfo osdata;

struct _irqinfo *
get_irq (id)
int id;
{
	static struct _irqinfo data;

	if (id < 0 || id >= osdata.num_handlers) {
		return (NULL);
	}

	if ((id = qnx_hint_query (PROC_PID, id, &data)) != -1) {
		last_irqid = id;
		return (&data);
	}

	return (NULL);
}

struct _psinfo *
get_proc (id)
pid_t id;
{
	static struct _psinfo data;

	if ((id = PINDEX (id)) <= 0 || id > osdata.num_procs) {
		return (NULL);
	}

	if (qnx_psinfo (PROC_PID, id, &data, 0, NULL) != -1) {
		return (&data);
	}

	return (NULL);
}

getIRQNames ()
{
	int intnum;
	register struct _irqinfo *ip;
	register struct _psinfo *pp;

	if (qnx_osinfo (0, &osdata) == -1) {
		fatal ("unable to connect to Proc");
	}

	for (intnum = -1 ; intnum < 16 ; ++intnum) {
		for (last_irqid = 0 ; ip = get_irq (last_irqid); ++last_irqid) {
			if (ip -> intnum == intnum
				&& (pp = get_proc (ip -> pid))
				&&  pp -> pid == ip -> pid) {
				if (intnum > 0) {
					if (!fnmatch ("*Fsys*", pp -> un.proc.name, 0)) {
						if (!fnmatch ("*floppy*", pp -> un.proc.name, 0)) {
							strcpy (irqNames [intnum], "FDsk");
						} else {
							strcpy (irqNames [intnum], "HDsk");
						}
					} else if (!fnmatch ("*ser*", pp -> un.proc.name, 0)) {
						strcpy (irqNames [intnum], "Ser ");
					} else if (!fnmatch ("*Net*", pp -> un.proc.name, 0)) {
						strcpy (irqNames [intnum], "Net ");
					}
				}
			}
		}
	}
}

