
/*
 *	main.c
 *
 *	QNX 4, MAIN.C shell version 0.003
 *
 *	(C) Copyright 1991, 1992, 1996 by Robert Krten, all rights reserved.
 *
 *	This module represents the main module for the system monitor
 *	(SYSMON) utility.
 *
 *	This program will display various system parameters graphically.
 *
 *	This utility is not intended for porting to other OS's.
 *
 *	This program accepts a command line consisting of the following
 *	options:
 *
 *		[a=attribute] [m=<monitorType>] [i=interval(seconds)] [+histogram]
 *
 *	1991 09 20	R. Krten		created
 *	1991 09 22	R. Krten		made node1 only, and released to QUICS
 *	1991 09 23	R. Krten		added i=interval stuff
 *	1991 11 03	R. Krten		fixed usage message to use stderr and \r's
 *	1992 01 05	R. Krten		released for free
 *	1992 04 18	R. Krten		added m=status
 *	1996 08 17	R. Krten		added kernel event info (m=interrupts)
 *	1996 08 17	R. Krten		change i=interval to be floating point
 *	1996 08 17	R. Krten		added faults and kcalls
*/

#include "sysmon.h"

static	void		usage (int);
static	void		optproc (int, char **);

char	*progname = "sysmon";
char	*blankname= "      ";
extern	char *version;					/* version.c */
extern	struct _osinfo osData;

int		opta;
int		opth;
int		optims;
int		optm;

void
main (argc, argv)
int		argc;
char	**argv;
{
	optproc (argc, argv);
	if (term_load () == -1) {
		printf ("%s:  error loading terminal info.\n", progname);
		perror (NULL);
		exit (1);
	}
	term_video_on ();
	term_clear (TERM_CLS_SCRH);
	term_printf (0, 0, opta, "SYSMON V%s, (C) Copyright 1991 by Robert Krten", version);
	term_flush ();
	switch (optm) {
	case	SMfaults:			doFAULTS ();				break;
	case	SMinterrupts:		doINTERRUPTS ();			break;
	case	SMkcalls:			doKCALLS ();				break;
	case	SMphysmem:			doPHYSMEM ();				break;
	case	SMregister:			doREGISTER ();				break;
	case	SMstates:			doSTATES ();				break;
	case	SMstatus:			doSTATUS ();				break;
	case	SMtopcode:			doTOPMEM ('c');				break;
	case	SMtopcpu:			doTOPCPU ();				break;
	case	SMtopdata:			doTOPMEM ('d');				break;
	case	SMtopmem:			doTOPMEM ('m');				break;
	}
	term_clear (TERM_CLS_SCRH);
	term_restore ();
}

void
fatal (msg)
char	*msg;
{
	term_clear (TERM_CLS_SCRH);
	term_restore ();
	printf ("\n%s:  FATAL ERROR:  %s\n\r", progname, msg);
	exit (1);
}

static void
usage (n)
int		n;
{
	system ("use sysmon");
	if (n) {
		exit (n);
	}
}

static void
optproc (argc, argv)
int		argc;
char	**argv;
{
	int		i;
	int		optFound;

	if (!argc) {
		usage (1);
	}
	opta = TERM_NORMAL;
	opth = 0;
	optims = 1000;
	optm = SMtopcpu;
	if (argc > 1) {
		for (i = 1; i < argc; i++) {
			optFound = 0;
			if (argv [i][0] == '-') {
				optFound = '-';
			} else if (argv [i][0] == '+') {
				optFound = '+';
			}
			if (optFound) {
				optFound = (optFound == '+');
				switch (argv [i][1]) {
				case	'h':
					opth = optFound;
					break;
				default:
					usage (1);
					break;
				}
			} else {
				if (argv [i][1] == '=') {
					switch (argv [i][0]) {
					case	'a':
						if (!sscanf (argv [i] + 2, "%x", &opta)) {
							printf ("%s:  invalid hex attribute.\n", progname);
							usage (1);
						}
						break;
					case	'i':
						optims = atof (argv [i] + 2) * 1000.;
						break;
						/*
						 *	select monitor type:
						 *		i		interrupts
						 *		r		registers
						 *		state	states
						 *		statu	status
						 *		topco	topcode
						 *		topcp	topcpu
						 *		topd	topdata
						 *		topm	topmem
						*/
					case	'm':
						if (!fnmatch ("f*", argv [i] + 2, 0)) {
							optm = SMfaults;
						} else if (!fnmatch ("i*", argv [i] + 2, 0)) {
							optm = SMinterrupts;
						} else if (!fnmatch ("k*", argv [i] + 2, 0)) {
							optm = SMkcalls;
						} else if (!fnmatch ("p*", argv [i] + 2, 0)) {
							optm = SMphysmem;
						} else if (!fnmatch ("r*", argv [i] + 2, 0)) {
							optm = SMregister;
						} else if (!fnmatch ("state*", argv [i] + 2, 0)) {
							optm = SMstates;
						} else if (!fnmatch ("statu*", argv [i] + 2, 0)) {
							optm = SMstatus;
						} else if (!fnmatch ("topco*", argv [i] + 2, 0)) {
							optm = SMtopcode;
						} else if (!fnmatch ("topcp*", argv [i] + 2, 0)) {
							optm = SMtopcpu;
						} else if (!fnmatch ("topd*", argv [i] + 2, 0)) {
							optm = SMtopdata;
						} else if (!fnmatch ("topm*", argv [i] + 2, 0)) {
							optm = SMtopmem;
						} else {
							printf ("%s:  unknown monitor type \"%s\"\n", progname, argv [i] + 2);
							usage (1);
						}
						break;
					default:
						usage (1);
						break;
					}
				} else {
					usage (1);
				}
			}
		}
	}
}

