
/*
 *	physmem.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1991, 1992 by Robert Krten, all rights reserved.
 *
 *	This module is responsible for the physical memory display.
 *
 *	1991 09 22	R. Krten		created
 *	1992 01 05	R. Krten		released for free
*/

#include "sysmon.h"

extern	char *progname;					/* main.c */
extern	char *version;					/* version.c */

extern	int	opta;
extern	int optims;						// millisecond sleep
extern	struct _psinfo psData;

struct _osinfo osData;

static	void clearScreenImage (void);
static	void accumulateMemory (void);
static	void markSegmentData (unsigned long, long, char);
static	void displayMemory (void);

static	struct _seginfo segData [16];
static	int kPerChar;
static	int nlines;
static	char *screenImage;
static	unsigned long totMemK;			/* total installed memory, kilobytes */

void
doPHYSMEM ()
{
	int		nchars;
	int		i;
	int		iter = 0;

	qnx_osinfo (0, &osData);
	totMemK = osData.totmemk;
	nchars = (nlines = (term_state.num_rows - 5)) * 70;
	kPerChar = totMemK / nchars;
	kPerChar++;		/* ==1 for 512k and 50 lines, ==12 for 16M and 25 lines */

	/*
	 *	set up full screen console stuff
	*/

	term_cur (1, 0);
	term_clear (TERM_CLS_EOS);
	term_printf (2, 21, opta, "Physical Memory Display (total installed %dk)", totMemK);
	term_printf (3, 0, opta, "        Ŀ");
	for (i = 4; i < (term_state.num_rows - 1); i++) {
		term_printf (i, 0, opta, "%08lX                                                                      ", (long) (i - 4) * (long) kPerChar * 1024L * 70L);
	}
	term_printf (i, 0, opta, "        ");
	term_flush ();

	if ((screenImage = malloc ((term_state.num_rows - 5) * 71)) == NULL) {
		fatal ("couldn't allocate screen image");
	}

	for (;;) {

		clearScreenImage ();
		accumulateMemory ();
		displayMemory ();
		term_printf (2, 70, opta, "(%d)", iter++);
		delay (optims);

		if (dev_ischars (0) && getc (stdin) == 'q') {
			break;
		}
	}
}

static void
clearScreenImage ()
{
	int		i, j;

	for (i = 0; i < nlines; i++) {
		for (j = 0; j < 70; j++) {
			screenImage [i * 71 + j] = ' ';
		}
		screenImage [i * 71 + j] = 0;
	}
}

static void
accumulateMemory ()
{
	pid_t	currentPID;
	int		sindex;
	char	letter;

	currentPID = 1;
	for (;;) {
		currentPID = qnx_psinfo (PROC_PID, currentPID, &psData, 0, segData);
		if (currentPID == -1) {
			if (errno == ESRCH || errno == EINVAL) {
				return;				/* done, go away */
			}
		}
		if ((psData.flags & (_PPF_MID | _PPF_VID)) == 0) {
			for (sindex = 0; sindex < psData.un.proc.nselectors; sindex++) {
				if (segData [sindex].flags & _PMF_GLOBAL) {
					letter = 'G';
				} else {
					switch (segData [sindex].flags & 0x03) {
					case	0:		/* Data, read and write */
						letter = 'W';
						break;
					case	1:		/* Data, read only */
						letter = 'R';
						break;
					case	2:		/* Code, read and execute */
						letter = 'C';
						break;
					case	3:		/* Code, execute only */
						letter = 'X';
						break;
					}
				}
				markSegmentData (segData [sindex].addr, segData [sindex].nbytes, letter);
			}
		}
		currentPID++;
	}
}

void
markSegmentData (phys, size, letter)
unsigned long	phys;
long	size;
char	letter;
{
	int		start, end;
	int		startx, starty;
	int		endx, endy;
	int		x, y;

	if ((phys / 1024L) > totMemK) {
		return;
	}

	start = (phys / ((long) kPerChar * 1024L));
	end = ((phys + size) / ((long) kPerChar * 1024L));

	starty = start / 70;
	startx = start % 70;
	endy = end / 70;
	endx = end % 70;

	for (y = starty; y < endy; y++) {
		for (x = (y == starty ? startx : 0); x < 70; x++) {
			screenImage [y * 71 + x] = letter;
		}
	}
	for (x = (y == starty ? startx : 0); x <= endx; x++) {
		screenImage [y * 71 + x] = letter;
	}
}

static void
displayMemory ()
{
	int		y;

	for (y = 0; y < nlines; y++) {
		term_printf (y + 4, 9, opta, "%s", &screenImage [y * 71]);
	}
	term_flush ();
}

