
/*
 *	register.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1991, 1992 by Robert Krten, all rights reserved.
 *
 *	This module is responsible for the REGISTER display.
 *
 *	1991 09 23	R. Krten		created
 *	1991 09 23	R. Krten		many thanks to DTDODGE for "sin.c"
 *	1992 01 05	R. Krten		released for free
*/

#include "sysmon.h"

#define	MaxHashEntries			16		/* hash is lowest four bits */

static	void accumulateRegister (int);
static	void removeDeadEntries (int);
static	void displayRegister (void);

extern	char *progname;					/* main.c */
extern	char *version;					/* version.c */
extern	int	opta;
extern	int	optims;
extern	struct _psinfo psData;
extern	struct _osinfo osData;

static	int numregisterLines;			/* 15 for a 25 line screen */
struct	_reg_struct regData;

static	RegisterT registerBackbone [MaxHashEntries];

void
doREGISTER ()
{
	int		i;

	/*
	 *	clear out the REGISTER backbone for the hash table
	*/

	for (i = 0; i < MaxHashEntries; i++) {
		registerBackbone [i].next = NULL;
	}

	/*
	 *	set up full screen console stuff
	*/

	numregisterLines = term_state.num_rows - 10;
	term_cur (1, 0);
	term_clear (TERM_CLS_EOS);
	term_printf (2, 21, opta, "REGISTER Display");
	term_printf (3, 0, opta, "%s", "Ŀ");
	term_printf (4, 0, opta, "%s", " Name   CS   IP  SS   SP  DS  ES  AX  BX  CX  DX  SI  DI  BP Flag");
	term_printf (5, 0, opta, "%s", "");
	term_flush ();

	if (!optims) {
		optims += 1000;		/* sleep (0) messes up __qnx_debug calls... */
	}

	for (;;) {
		delay (optims);
		accumulateRegister (1);
		removeDeadEntries (1);
		displayRegister ();

		delay (optims);
		accumulateRegister (2);
		removeDeadEntries (2);
		displayRegister ();

		if (dev_ischars (0) && getc (stdin) == 'q') {
			break;
		}
	}
}

static void
accumulateRegister (pass)
int		pass;
{
	pid_t	currentPID;
	RegisterT	*ptr;
	RegisterT	*prev;

	currentPID = 1;
	for (;;) {
		currentPID = qnx_psinfo (PROC_PID, currentPID, &psData, 0, NULL);
		if (currentPID == -1) {
			if (errno == ESRCH || errno == EINVAL) {
				return;				/* done, go away */
			}
		}
		if ((psData.flags & (_PPF_MID | _PPF_VID)) == 0) {
			prev = &registerBackbone [currentPID & 0x0f];
			for (ptr = prev -> next; ptr != NULL; ptr = (prev = ptr) -> next) {
				if (ptr -> pid == currentPID) {
					break;
				}
			}
			if (ptr == NULL) {
				if ((prev -> next = (RegisterT *) malloc (RegisterS)) == NULL) {
					fatal ("Couldn't allocate memory for new register hash entry");
				}
				ptr = prev -> next;		/* get to new entry */
				ptr -> pid = currentPID;
				fixName (ptr -> name, psData.un.proc.name);
				ptr -> next = NULL;
			}
			ptr -> pass = pass;
			if (__qnx_debug_xfer (PROC_PID, currentPID, _DEBUG_REG_RD, &(ptr -> regs), sizeof (struct _reg_struct), 0, 0) == -1) {
				ptr -> valid = 0;
			} else {
				ptr -> valid = 1;
			}
		}
		currentPID++;
	}
}

static void
removeDeadEntries (pass)
int		pass;
{
	int		i;
	RegisterT	*ptr;
	RegisterT	*prev;

	for (i = 0; i < MaxHashEntries; i++) {
		prev = &registerBackbone [i];
		ptr = prev -> next;
		while (ptr != NULL) {
			if (!*ptr -> name) {
				getName (ptr -> name, ptr -> pid);
			}
			if (ptr -> pass != pass) {
				prev -> next = ptr -> next;
				free (ptr);
			} else {
				prev = ptr;
			}
			ptr = prev -> next;
		}
	}
}

static void
displayRegister ()
{
	int		i;
	int		p;
	RegisterT *ptr;

	p = 6;
	for (i = 0; i < MaxHashEntries; i++) {
		ptr = registerBackbone [i].next;
		while (ptr != NULL) {
			term_printf (p, 0, opta, "%-8.8s %04lX:%04lX %04lX:%04lX %04lX %04lX %04lX %04lX %04lX %04lX %04lX %04lX %04lX %04lX",
				ptr -> name,
				ptr -> regs.cs, ptr -> regs.ip, ptr -> regs.ss, ptr -> regs.sp,
				ptr -> regs.ds, ptr -> regs.es,
				ptr -> regs.ax, ptr -> regs.bx, ptr -> regs.cx, ptr -> regs.dx,
				ptr -> regs.si, ptr -> regs.di, ptr -> regs.bp, ptr -> regs.fl);
			ptr = ptr -> next;
			if (p++ > numregisterLines) {
				break;
			}
		}
	}
	if (p < numregisterLines) {
		term_cur (p, 0);
		term_clear (TERM_CLS_EOS);
	}
	term_flush ();
}

