
/*
 *	states.c
 *
 *	QNX
 *
 *	(C) Copyright 1991, 1992 by Robert Krten, all rights reserved.
 *
 *	This module is responsible for monitoring process states.
 *
 *	1991 09 21	R. Krten		created
 *	1992 01 05	R. Krten		released for free
*/

#include "sysmon.h"

extern	char *progname;					/* main.c */
extern	char *version;					/* version.c */

static	void	stateMonitor (void);
static	void	totalStates (void);

extern	int opta;
extern	int	optims;
extern	struct _psinfo psData;
extern	char *lines;

void
doSTATES ()
{
	term_cur (1, 0);
	term_clear (TERM_CLS_EOS);
	term_printf (2, 21, opta, "CPU Process States");
	term_printf (4,  0, opta, "Ŀ");
	term_printf (5,  0, opta, " State    Count                                                            ");
	term_printf (6,  0, opta, "Ĵ");
	term_printf (7,  0, opta, " DEAD           ");
	term_printf (8,  0, opta, "Ĵ");
	term_printf (9,  0, opta, " READY          ");
	term_printf (10, 0, opta, "Ĵ");
	term_printf (11, 0, opta, " SEND           ");
	term_printf (12, 0, opta, "Ĵ");
	term_printf (13, 0, opta, " RECEIVE        ");
	term_printf (14, 0, opta, "Ĵ");
	term_printf (15, 0, opta, " REPLY          ");
	term_printf (16, 0, opta, "Ĵ");
	term_printf (17, 0, opta, " HELD           ");
	term_printf (18, 0, opta, "Ĵ");
	term_printf (19, 0, opta, " SIGNAL         ");
	term_printf (20, 0, opta, "Ĵ");
	term_printf (21, 0, opta, " WAIT           ");
	term_printf (22, 0, opta, "Ĵ");
	term_printf (23, 0, opta, " Total          ");
	term_printf (24, 0, opta, "");
	term_flush ();
	
	for (;;) {
		stateMonitor ();
		delay (optims);

		if (dev_ischars (0) && getc (stdin) == 'q') {
			break;
		}
	}
}

static	int		stateCount [9];
static	char	buf [20];

static void
stateMonitor ()
{
	int		i;
	int		percentage;

	accumulateStates ();
	for (i = 0; i < 9; i++) {
		percentage = ((float) stateCount [i] * 100. / (float) stateCount [8]);
		term_printf (i * 2 + 7, 12, opta, "%5d", stateCount [i]);
		sprintf (buf, "%%%d.%ds", percentage / 2, percentage / 2);
		if (stateCount [i]) {
			term_printf (i * 2 + 7, 20, opta, buf, lines);
		} else {
			term_cur (i * 2 + 7, 20);
		}
		term_clear (TERM_CLS_EOL);
		term_printf (i * 2 + 7, 78, opta, "");
	}
	term_flush ();
}

accumulateStates ()
{
	pid_t	currentPID;
	int		i;

	for (i = 0; i < 8; i++) {
		stateCount [i] = 0;
	}

	currentPID = 1;
	for (;;) {
		currentPID = qnx_psinfo (PROC_PID, currentPID, &psData, 0, NULL);
		if (currentPID == -1) {
			if (errno == ESRCH || errno == EINVAL) {
				totalStates ();
				return;				/* done, go away */
			}
		}
		if ((psData.flags & (_PPF_MID | _PPF_VID)) == 0) {
			stateCount [psData.state & 7]++;
		}
		currentPID++;
	}
}

void
totalStates ()
{
	int		i;

	stateCount [8] = 0;
	for (i = 0; i < 8; i++) {
		stateCount [8] += stateCount [i];
	}
}

