
/*
 *	status.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1992 by Robert Krten, all rights reserved.
 *
 *	This module is responsible for the process status summary display.
 *
 *	1992 04 18	R. Krten		created
*/

#include "sysmon.h"

void	accumulateSTATUSBackbone (int);
int		compareSTATUS (StatusT **, StatusT **);
void	displaySTATUS (void);
void	displaySTATUSHistogram (void);
void	fetchSortSTATUS (void);
void	removeDeadSTATUSEntries (int);

#define	MaxHashEntries			16		/* hash is lowest four bits */
extern	char *progname;					/* main.c */
extern	char *version;					/* version.c */

extern	int	opta;
extern	int	opth;
extern	int	optims;

extern struct _osinfo osData;
extern struct _psinfo psData;
extern struct _seginfo segData [16];
extern char *lines;

static	int numStatusLines;				/* 15 for a 25 line screen */
static	int processCount;
static	int last80 [80];
static	long currentHistogramValue;
static	long histogramTotal;
static	long histogramSamples;

static	StatusT	statusBackbone [MaxHashEntries];

static	char *stateNames [8] =
{
	"DEAD",	"READY", "SEND", "RECVE", "REPLY", "HELD", "SIGNL", "WAIT"
};

void
doSTATUS ()
{
	int		i;
	int		histogramCount;

	/*
	 *	clear out the status backbone for the hash table, and initialize
	*/

	for (i = 0; i < MaxHashEntries; i++) {
		statusBackbone [i].next = NULL;
	}
	processCount = 0;
	for (i = 0; i < 80; i++) {
		last80 [i] = 0;
	}


	/*
	 *	set up full screen console stuff
	*/

	numStatusLines = term_state.num_rows - 23;
	term_cur (1, 0);
	term_clear (TERM_CLS_EOS);
	term_printf (2, 32, opta, "Process Summary");
	term_printf (3, 0, opta, "%s", "Ŀ");
	term_printf (4, 0, opta, "%s", " PID       Name       Pri  State  %  Accumulated CPU  Code k  Data k ");
	term_printf (5, 0, opta, "%s", "Ĵ");

	/*
	 *	start it up!
	*/

	accumulateSTATUSBackbone (1);
	histogramCount = 0;
	histogramTotal = 0;
	histogramSamples = 0;
	currentHistogramValue = 0;
	if (opth) {
		displaySTATUSHistogram ();		/* just to get titles, etc */
	}
	histogramSamples = 0;
	for (;;) {
		delay (optims);
		accumulateSTATUSBackbone (2);
		removeDeadSTATUSEntries (2);
		displaySTATUS ();

		delay (optims);
		accumulateSTATUSBackbone (1);
		removeDeadSTATUSEntries (1);
		displaySTATUS ();

		if (histogramCount++ >= 24) {
			if (opth) {
				displaySTATUSHistogram ();
				currentHistogramValue = 0;
			}
			histogramCount = 0;
		}

		if (dev_ischars (0) && getc (stdin) == 'q') {
			break;
		}
	}
}

int
compareSTATUS (a, b)
StatusT **a;
StatusT	**b;
{
	if ((*a) -> pid < (*b) -> pid) {
		return (-1);
	} else if ((*a) -> pid > (*b) -> pid) {
		return (1);
	} else {
		return (0);
	}
}

static StatusT **processList;

void
fetchSortSTATUS ()
{
	int		i;
	int		p;
	StatusT *ptr;

	if ((processList = (StatusT **) malloc (sizeof (StatusT *) * processCount)) == NULL) {
		fatal ("Couldn't allocate memory for new processList");
	}
	p = 0;
	for (i = 0; i < MaxHashEntries; i++) {
		ptr = statusBackbone [i].next;
		while (ptr != NULL) {
			processList [p++] = ptr;
			ptr = ptr -> next;
		}
	}
	qsort (processList, p, sizeof (StatusT *), compareSTATUS);
}

void
displaySTATUS ()
{
	int		i;
	clock_t	grandTotal;
	long	totHuns;
	int		d, h, m, s, hun;
	int		percentage;

	fetchSortSTATUS ();
	grandTotal = 0;
	for (i = 0; i < processCount; i++) {
		grandTotal += processList [i] -> delta;
	}
	for (i = 0; (i < processCount) && (i < numStatusLines); i++) {
		term_printf (6 + i, 0, opta, "%05d%-16.16s", processList [i] -> pid, processList [i] -> name);
		term_clear (TERM_CLS_EOL);
		term_printf (6 + i, 29, opta, " %5.5s", stateNames [processList [i] -> state]);
		percentage = (int) ((float) processList [i] -> delta * 100. / (float) grandTotal);
		term_printf (6 + i, 37, opta, "%3d%%", percentage);
		totHuns = processList [i] -> total / ((long) CLK_TCK / 100L);
		d = totHuns / (360000L * 24L);
		h = (totHuns -= (long) d * 360000L * 24L) / 360000L;
		m = (totHuns -= (long) h * 360000L) / 6000L;
		s = (totHuns -= (long) m * 6000L) / 100L;
		hun = totHuns - s * 100L;
		term_printf (6 + i, 41, opta, " %3d %02d:%02d:%02d.%02d", d, h, m, s, hun);
		term_printf (6 + i, 59, opta, " %6ld ", processList [i] -> codeK);
		term_printf (6 + i, 68, opta, " %6ld ", processList [i] -> dataK);
	}
	term_printf (6 + i, 0, opta, "%s", "");
	if (opth) {
		for (; i < (numStatusLines - 1); i++) {
			term_cur (7 + i, 0);
			term_clear (TERM_CLS_EOL);
		}
	} else {
		if (i < (numStatusLines - 1)) {
			term_cur (7 + i, 0);
			term_clear (TERM_CLS_EOS);
		}
	}
	term_flush ();
	free (processList);
}

void
accumulateSTATUSBackbone (pass)
int		pass;
{
	pid_t	currentPID;
	int		sindex;
	StatusT	*ptr;
	StatusT	*prev;
	int		segmentIndex;

	currentPID = 1;
	for (;;) {
		currentPID = qnx_psinfo (PROC_PID, currentPID, &psData, 0, NULL);
		if (currentPID == -1) {
			if (errno == ESRCH || errno == EINVAL) {
				return;				/* done, go away */
			}
		}
		if ((psData.flags & (_PPF_MID | _PPF_VID)) == 0) {
			prev = &statusBackbone [currentPID & 0x0f];
			for (ptr = prev -> next; ptr != NULL; ptr = (prev = ptr) -> next) {
				if (ptr -> pid == currentPID) {
					ptr -> prio = psData.priority;
					ptr -> delta = ptr -> total;
					ptr -> total = psData.un.proc.times.tms_cutime;
					ptr -> total+= psData.un.proc.times.tms_cstime;
					ptr -> total+= psData.un.proc.times.tms_stime;
					ptr -> total+= psData.un.proc.times.tms_utime;
					ptr -> delta = ptr -> total - ptr -> delta;
					ptr -> state = psData.state & 7;
					ptr -> dataK = ptr -> codeK = 0;
					for (segmentIndex = 0; qnx_psinfo (PROC_PID, currentPID, &psData, segmentIndex, &segData) == currentPID; segmentIndex += 16) {
						if (psData.un.proc.nselectors == 0) {
							break;
						}
						for (sindex = 0; sindex < psData.un.proc.nselectors; sindex++) {
							if (segData [sindex].flags & _PMF_INUSE) {
								if ((segData [sindex].flags & _PMF_BORROWED) == 0) {
									continue;
								}
								if ((segData [sindex].flags & (_PMF_SHARED | _PMF_LINKED)) == (_PMF_SHARED | _PMF_LINKED)) {
									/* shared_bytes += segData [sindex].nbytes; */
								} else if (segData [sindex].flags & _PMF_CODE_RX) {
									ptr -> codeK += segData [sindex].nbytes;
								} else {
									ptr -> dataK += segData [sindex].nbytes;
								}
							}
						}
					}
					ptr -> codeK >>= 10;		/* convert to k */
					ptr -> dataK >>= 10;		/* convert to k */
					break;
				}
			}
			if (ptr == NULL) {
				if ((prev -> next = (StatusT *) malloc (StatusS)) == NULL) {
					fatal ("Couldn't allocate memory for new StatusT hash entry");
				}
				ptr = prev -> next;		/* get to new entry */
				ptr -> pid = currentPID;
				ptr -> prio = psData.priority;
				fixName (ptr -> name, psData.un.proc.name);
				ptr -> total = psData.un.proc.times.tms_cutime;
				ptr -> total+= psData.un.proc.times.tms_cstime;
				ptr -> total+= psData.un.proc.times.tms_stime;
				ptr -> total+= psData.un.proc.times.tms_utime;
				ptr -> delta = 0;	/* new processes don't have any time! */
				ptr -> state = psData.state & 7;
				ptr -> next = NULL;
				ptr -> codeK = ptr -> dataK = 0;
				processCount++;
			}
			ptr -> pass = pass;
		}
		currentPID++;
	}
}

void
removeDeadSTATUSEntries (pass)
int		pass;
{
	int		i;
	StatusT	*ptr;
	StatusT	*prev;

	for (i = 0; i < MaxHashEntries; i++) {
		prev = &statusBackbone [i];
		ptr = prev -> next;
		while (ptr != NULL) {
			if (!*ptr -> name) {
				getName (ptr -> name, ptr -> pid);
			}
			if (ptr -> pass != pass) {
				prev -> next = ptr -> next;
				free (ptr);
				processCount--;
			} else {
				prev = ptr;
			}
			ptr = prev -> next;
		}
	}
}

static char buf [80];

/*
 *	displayHistogram ()
 *
 *	This routine is responsible for displaying the histogram on the bottom
 *	of the screen.
*/

void
displaySTATUSHistogram ()
{
	int		x;
	int		y;

	for (x = 0; x < 79; x++) {
		buf [x] = 0xdb;			/* fill with full bar */
	}
	buf [79] = 0;
	for (x = 0; x < 78; x++) {
		last80 [x] = last80 [x + 1];
	}
	last80 [78] = currentHistogramValue / 167L;
	for (y = 1; y < 30; y += 2) {
		for (x = 1; x < 79; x++) {
			if (last80 [x] == y) {
				buf [x] = 0xdc;				/* only get half a bar */
			} else if (last80 [x] < y) {
				buf [x] = ' ';
			}
		}
		term_printf (term_state.num_rows - (y / 2) - 2, 1, opta, "%s", buf + 1);
	}
	term_printf (term_state.num_rows - 2, 0, opta, "0");
	term_printf (term_state.num_rows - 16, 0, opta, "1");
	term_printf (term_state.num_rows - 9, 0, opta, "");
	histogramSamples++;
	histogramTotal += currentHistogramValue / 50L;
	term_printf (term_state.num_rows - 1, 0, opta, "Histogram sample %ld, average CPU utilization %2d%%", histogramSamples, (int) (histogramTotal / histogramSamples));
	term_flush ();
}

