
/*
 *	topmem.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1991, 1992 by Robert Krten, all rights reserved.
 *
 *	This module is responsible for the TOP MEM (code, data, and memory)
 *	display.
 *
 *	1991 09 22	R. Krten		created
 *	1992 01 05	R. Krten		released for free
*/

#include "sysmon.h"

static	void	accumulateBackbone (int);
static	void	removeDeadEntries (int);
static	int		compareTOPMEM (TopmemT **, TopmemT **);
static	void	fetchSortTOPMEM (void);
static	void	displayDeltaTOPMEM (void);

#define	MaxHashEntries			16		/* hash is lowest four bits */
extern	char *progname;					/* main.c */
extern	char *version;					/* version.c */

extern	int	opta;
extern	int	optims;

extern	struct _psinfo psData;
extern	struct _osinfo osData;
extern	char *lines;

struct _seginfo segData [16];

char	*toplines = "";
char	*botlines = "";

static	int numtopmemLines;				/* 15 for a 25 line screen */
static	int processCount;
static	unsigned long totMemK;			/* total installed memory, kilobytes */

static	TopmemT	topmemBackbone [MaxHashEntries];

static	int	displayType;

void
doTOPMEM (type)
char	type;
{
	int		i;

	/*
	 *	clear out the top MEM backbone for the hash table
	*/

	for (i = 0; i < MaxHashEntries; i++) {
		topmemBackbone [i].next = NULL;
	}
	processCount = 0;
	qnx_osinfo (0, &osData);
	totMemK = osData.totmemk;
	displayType = type;

	/*
	 *	set up full screen console stuff
	*/

	numtopmemLines = term_state.num_rows - 10;
	term_cur (1, 0);
	term_clear (TERM_CLS_EOS);
	term_printf (3, 0, opta, "%s", "Ŀ");
	term_printf (5, 0, opta, "%s", "Ĵ");
	switch (type) {
	case	'c':		/* code only */
		term_printf (2, 21, opta, "TOP CODE MEMORY (total installed %dk)", totMemK);
		term_printf (4, 0, opta, "%s", " PID       Name       %                                                  ");
		break;
	case	'd':		/* data only */
		term_printf (2, 21, opta, "TOP DATA MEMORY (total installed %dk)", totMemK);
		term_printf (4, 0, opta, "%s", " PID       Name       %                                                  ");
		break;
	case	'm':		/* both */
		term_printf (2, 21, opta, "TOP MEMORY (total installed %dk)", totMemK);
		term_printf (4, 0, opta, "%s", " PID       Name       %  CODE             DATA                 ");
		break;
	}

	for (;;) {
		delay (optims);
		accumulateMemory (1);
		removeDeadEntries (1);
		displayTOPMEM ();

		delay (optims);
		accumulateMemory (2);
		removeDeadEntries (2);
		displayTOPMEM ();

		if (dev_ischars (0) && getc (stdin) == 'q') {
			break;
		}
	}
}

static void
accumulateMemory (pass)
int		pass;
{
	pid_t	currentPID;
	TopmemT	*ptr;
	TopmemT	*prev;
	int		sindex;
	int		segmentIndex;

	currentPID = 1;
	for (;;) {
		currentPID = qnx_psinfo (PROC_PID, currentPID, &psData, 0, segData);
		if (currentPID == -1) {
			if (errno == ESRCH || errno == EINVAL) {
				return;				/* done, go away */
			}
		}
		if ((psData.flags & (_PPF_MID | _PPF_VID)) == 0) {
			prev = &topmemBackbone [currentPID & 0x0f];
			for (ptr = prev -> next; ptr != NULL; ptr = (prev = ptr) -> next) {
				if (ptr -> pid == currentPID) {
					break;
				}
			}
			if (ptr == NULL) {
				if ((prev -> next = (TopmemT *) malloc (TopmemS)) == NULL) {
					fatal ("Couldn't allocate memory for new topmem hash entry");
				}
				ptr = prev -> next;		/* get to new entry */
				ptr -> pid = currentPID;
				fixName (ptr -> name, psData.un.proc.name);
				ptr -> next = NULL;
				processCount++;
			}
			ptr -> pass = pass;
			ptr -> dataK = ptr -> codeK = 0;
			for (segmentIndex = 0; qnx_psinfo (PROC_PID, currentPID, &psData, segmentIndex, &segData) == currentPID; segmentIndex += 16) {
				if (psData.un.proc.nselectors == 0) {
					break;
				}
				for (sindex = 0; sindex < psData.un.proc.nselectors; sindex++) {
					if (segData [sindex].flags & _PMF_INUSE) {
						if ((segData [sindex].flags & _PMF_BORROWED) == 0) {
							continue;
						}
						if (segData [sindex].nbytes / 1024 > totMemK) {
							continue;
						}
						if ((segData [sindex].flags & (_PMF_SHARED | _PMF_LINKED)) == (_PMF_SHARED | _PMF_LINKED)) {
							/* shared_bytes += segData [sindex].nbytes; */
						} else if (segData [sindex].flags & _PMF_CODE_RX) {
							ptr -> codeK += segData [sindex].nbytes;
						} else {
							ptr -> dataK += segData [sindex].nbytes;
						}
					}
				}
			}
			ptr -> codeK >>= 10;		/* convert to k */
			ptr -> dataK >>= 10;		/* convert to k */
		}
		currentPID++;
	}
}

static void
removeDeadEntries (pass)
int		pass;
{
	int		i;
	TopmemT	*ptr;
	TopmemT	*prev;

	for (i = 0; i < MaxHashEntries; i++) {
		prev = &topmemBackbone [i];
		ptr = prev -> next;
		while (ptr != NULL) {
			if (!*ptr -> name) {
				getName (ptr -> name, ptr -> pid);
			}
			if (ptr -> pass != pass) {
				prev -> next = ptr -> next;
				free (ptr);
				processCount--;
			} else {
				prev = ptr;
			}
			ptr = prev -> next;
		}
	}
}

static int
compareTOPMEM (a, b)
TopmemT **a;
TopmemT	**b;
{
	switch (displayType) {
	case	'c':
		return ((*b) -> codeK - (*a) -> codeK);
	case	'd':
		return ((*b) -> dataK - (*a) -> dataK);
	case	'm':
		return (((*b) -> codeK + (*b) -> dataK) - ((*a) -> codeK + (*a) -> dataK));
	}
	return (0);	/* to keep compiler happy */
}

static TopmemT **processList;
static TopmemT freeMem;

static void
fetchSortTOPMEM ()
{
	int		i;
	int		p;
	TopmemT *ptr;

	if ((processList = (TopmemT **) malloc (sizeof (TopmemT *) * (processCount+1))) == NULL) {
		fatal ("Couldn't allocate memory for new processList");
	}
	qnx_osinfo (0, &osData);
	strcpy (freeMem.name, "<free memory>");
	freeMem.pid = 0;
	freeMem.codeK = freeMem.dataK = osData.freememk;
	if (displayType == 'm') {
		freeMem.codeK /= 2;	/* share free memory between data and code */
		freeMem.dataK /= 2;
	}
	processList [0] = &freeMem;
	p = 1;
	for (i = 0; i < MaxHashEntries; i++) {
		ptr = topmemBackbone [i].next;
		while (ptr != NULL) {
			processList [p++] = ptr;
			ptr = ptr -> next;
		}
	}
	qsort (processList, p, sizeof (TopmemT *), compareTOPMEM);
}

/* Debug line {FILE *fp;int j; fp=fopen("debug.out","a");for(j=0;j<i;j++){fprintf(fp,"%s code %ld data %ld (bytes)\n",processList[j]->name,processList[j]->codeK,processList[j]->dataK);}fclose(fp);} */

static char procname [17];

static void
displayTOPMEM ()
{
	fetchSortTOPMEM ();
	switch (displayType) {
	case	'c':
		displayTOPMEMCode ();
		break;
	case	'd':
		displayTOPMEMData ();
		break;
	case	'm':
		displayTOPMEMMemory ();
		break;
	}
}

displayTOPMEMCode ()
{
	int		i;
	long	grandTotal;
	int		percentage;

	grandTotal = totMemK;
	for (i = 0; (i < processCount) && (i < numtopmemLines); i++) {
		if (!processList [i] -> codeK) {
			break;
		}
		term_printf (6 + i, 0, opta, "%05d%-16.16s", processList [i] -> pid, processList [i] -> name);
		term_clear (TERM_CLS_EOL);
		percentage = (int) ((float) processList [i] -> codeK * 100. / (float) grandTotal);
		sprintf (procname, "%%3d%%%d.%ds", percentage / 2, percentage / 2);
		term_printf (6 + i, 24, opta, procname, percentage, lines);
		term_printf (6 + i, 77, opta, "");
	}
	term_printf (6 + i, 0, opta, "%s", "");
	if (i < (numtopmemLines - 1)) {
		term_cur (7 + i, 0);
		term_clear (TERM_CLS_EOS);
	}
	term_flush ();
	free (processList);
}

displayTOPMEMData ()
{
	int		i;
	long	grandTotal;
	int		percentage;

	grandTotal = totMemK;
	for (i = 0; (i < processCount) && (i < numtopmemLines); i++) {
		if (!processList [i] -> dataK) {
			break;
		}
		term_printf (6 + i, 0, opta, "%05d%-16.16s", processList [i] -> pid, processList [i] -> name);
		term_clear (TERM_CLS_EOL);
		percentage = (int) ((float) processList [i] -> dataK * 100. / (float) grandTotal);
		sprintf (procname, "%%3d%%%d.%ds", percentage / 2, percentage / 2);
		term_printf (6 + i, 24, opta, procname, percentage, lines);
		term_printf (6 + i, 77, opta, "");
	}
	term_printf (6 + i, 0, opta, "%s", "");
	if (i < (numtopmemLines - 1)) {
		term_cur (7 + i, 0);
		term_clear (TERM_CLS_EOS);
	}
	term_flush ();
	free (processList);
}

displayTOPMEMMemory ()
{
	int		i;
	long	grandTotal;
	int		percentageCode;
	int		percentageData;
	int		percentage;

	grandTotal = totMemK;
	for (i = 0; (i < processCount) && (i < numtopmemLines); i++) {
		if (!processList [i] -> codeK && !processList [i] -> dataK) {
			break;
		}
		term_printf (6 + i, 0, opta, "%05d%-16.16s", processList [i] -> pid, processList [i] -> name);
		term_clear (TERM_CLS_EOL);
		percentageCode = (int) ((float) processList [i] -> codeK * 100. / (float) grandTotal);
		percentageData = (int) ((float) processList [i] -> dataK * 100. / (float) grandTotal);
		percentage = min (percentageData, percentageCode);
		sprintf (procname, "%%3d%%%d.%ds", percentage / 2, percentage / 2);
		term_printf (6 + i, 24, opta, procname, percentageCode + percentageData, lines);
		if (percentageData > percentage) {	/* .:. we have more data to display */
			percentage = (percentageData - percentage) / 2;
			sprintf (procname, "%%%d.%ds", percentage, percentage);
			term_printf (6 + i, 28 + percentageCode / 2, opta, procname, botlines);
		} else {
			percentage = (percentageCode - percentage) / 2;
			sprintf (procname, "%%%d.%ds", percentage, percentage);
			term_printf (6 + i, 28 + (percentageData) / 2, opta, procname, toplines);
		}
		term_printf (6 + i, 77, opta, "");
	}
	term_printf (6 + i, 0, opta, "%s", "");
	if (i < (numtopmemLines - 1)) {
		term_cur (7 + i, 0);
		term_clear (TERM_CLS_EOS);
	}
	term_flush ();
	free (processList);
}

